#
# Copyright 2018 Pixar
#
# Licensed under the terms set forth in the LICENSE.txt file available at
# https://openusd.org/license.
#
"""
Trace -- Utilities for counting and recording events.
"""

from pxr import Tf
Tf.PreparePythonModule()
del Tf

import contextlib

@contextlib.contextmanager
def TraceScope(label):
    """A context manager that calls BeginEvent on the global collector on enter 
    and EndEvent on exit."""
    try:
        collector = Collector()
        eventId = collector.BeginEvent(label)
        yield
    finally:
        collector.EndEvent(label)

del contextlib

def TraceFunction(obj):
    """A decorator that enables tracing the function that it decorates.
    If you decorate with 'TraceFunction' the function will be traced in the
    global collector."""
    
    collector = Collector()
    
    def decorate(func):
        import inspect

        if inspect.ismethod(func):
            callableTypeLabel = 'method'
            classLabel = func.__self__.__class__.__name__+'.'
        else:
            callableTypeLabel = 'func'
            classLabel = ''

        module = inspect.getmodule(func)
        if module is not None:
            moduleLabel = module.__name__+'.'
        else:
            moduleLabel = ''

        label = 'Python {0}: {1}{2}{3}'.format(
            callableTypeLabel,
            moduleLabel,
            classLabel,
            func.__name__)
        def invoke(*args, **kwargs):
            with TraceScope(label):
                return func(*args, **kwargs)

        invoke.__name__ = func.__name__
        # Make sure wrapper function gets attributes of wrapped function.
        import functools
        return functools.update_wrapper(invoke, func)

    return decorate(obj)

def TraceMethod(obj):
    """A convenience.  Same as TraceFunction but changes the recorded
    label to use the term 'method' rather than 'function'."""
    return TraceFunction(obj)



#
# Copyright 2025 Pixar
#
# Licensed under the terms set forth in the LICENSE.txt file available at
# https://openusd.org/license.
#

from .qt import QtWidgets, QtGui, QtCore
from pxr import Usd, Ts, Gf

"""
SplineViewer is a widget that visualizes a spline value of a UsdAttribute.
It samples the spline and draws it in a 2D coordinate system, with time on the
x-axis and value on the y-axis. It also shows a playhead for the current frame
and the current value at that frame.
"""
class SplineViewer(QtWidgets.QWidget):

    ### Some constants for the widget
    MIN_WIDTH = 400
    MIN_HEIGHT = 300
    MIN_TIME_RANGE = 20
    DEFAULT_SAMPLING_VALUESCALE = 1
    DEFAULT_SAMPLING_TOLERANCE = 1
    MIN_DELTA = 1e-5
    MARGIN_BUFFER = 10
    BACKGROUND_COLOR = QtGui.QColor(30, 30, 30)
    GRID_COLOR = QtGui.QColor(80, 80, 80)
    AXIS_COLOR = QtGui.QColor(200, 200, 200)
    SPLINE_COLOR = QtGui.QColor(0, 200, 255)
    KNOT_COLOR = QtGui.QColor(255, 255, 255)
    PLAYHEAD_COLOR = QtGui.QColor(255, 255, 0)
    KNOT_SIZE = 5
    TANGENT_SIZE = 3
    AXIS_TICK_LENGTH = 5
    AXIS_TICK_X_LABEL_WIDTH_OFFSET = 10
    AXIS_TICK_X_LABEL_HEIGHT_OFFSET = 15
    AXIS_TICK_Y_LABEL_WIDTH_OFFSET = 5
    AXIS_TICK_Y_LABEL_HEIGHT_OFFSET = 5

    def __init__(self, attr=None, parent=None, currentFrame=None):
        super().__init__(parent)
        self.setMinimumSize(self.MIN_WIDTH, self.MIN_HEIGHT)
        self.polylinesToPlot = None
        self.knots = None
        self.margin = 20
        self.numTicks = 5
        self.fontMetrics = QtGui.QFontMetrics(self.font())
        self.startTime = None
        self.endTime = None
        self.attr = None
        if attr:
            self.setWindowTitle(f"Spline Viewer: {attr.GetPath()}")
            self.setWindowFlags(QtCore.Qt.WindowType.Window)
            self.SetAttribute(attr, currentFrame)
    
    def SetAttribute(self, attr, frame):
        """
        Used by AttributeValueEditor to set the attribute and current frame.
        """
        if not attr or not isinstance(attr, Usd.Attribute):
            raise ValueError("Invalid attribute provided to SplineViewer.")

        if self.attr == attr:
            # If the attribute is the same, just update the current frame.
            self.currentFrame = frame
            self.update()
            return
        self.attr = attr
        self.currentFrame = frame

        # Set the start and end time based on the stage's time code range
        # to begin with.
        if self.startTime is None:
            self.startTime = attr.GetPrim().GetStage().GetStartTimeCode()
        if self.endTime is None:
            self.endTime = attr.GetPrim().GetStage().GetEndTimeCode()
        if self.endTime <= self.startTime:
            self.endTime = self.startTime + self.MIN_TIME_RANGE

        self._updateKnots()
        self._updateSamples()
        self._transformToDevice()
        self.update()

    def SetCurrentFrame(self, frame):
        """
        Used by AttributeValueEditor and the floating SplineViewer instances to 
        set the current frame when dataModel's currentFrame changes.
        """
        self.currentFrame = frame
        self.update()

    def SetStartAndEndTime(self, startTime, endTime):
        """
        Used by AttributeValueEditor and the floating SplineViewer instances to
        set the start and end time when dataModel's frame range changes
        explicitly.
        """
        if self.startTime != startTime or self.endTime != endTime:
            # The times are changing. Update our knots and samples.
            self.startTime = startTime
            self.endTime = endTime
            if self.endTime <= self.startTime:
                self.endTime = self.startTime + self.MIN_TIME_RANGE

            self._updateKnots()
            self._updateSamples()

        self._transformToDevice()
        self.update()

    def CanView(self, attr):
        """
        Used by AttributeValueEditor to determine if the attribute can be viewed
        by SplineViewer.
        """
        # Check if the attribute is a spline
        if not isinstance(attr, Usd.Attribute):
            return False

        if not attr.HasAuthoredValue():
            return False

        # Check if the value source is a spline
        return attr.HasSpline()

    def Clear(self):
        """
        Used by AttributeValueEditor to clear the SplineViewer, when prim
        selection changes.
        """
        # Clear the current attribute and reset the view
        self.attr = None
        self.currentFrame = None
        self.polylinesToPlot = None
        self.update()

    def _updateKnots(self):
        self.knots = None
        if not self.attr:
            return
        spline = self.attr.GetSpline()
        if not spline or spline.IsEmpty():
            return
        timeInterval = Gf.Interval(self.startTime, self.endTime)
        self.knots = spline.GetKnotsWithLoopsBaked(timeInterval).values()

    def _getEstimatedMinMaxFromSpline(self, spline):
        # TODO: Use knot tangents to estimate min/max more accurately
        values = []
        prevCurve = False
        for knot in self.knots:
            t = knot.GetTime()

            if self.startTime <= t:
                pv = float(knot.GetPreValue())
                values.append(pv)
                # Only include tangents if we would draw them
                if prevCurve:
                    values.append(
                        pv - knot.GetPreTanWidth() * float(knot.GetPreTanSlope()))

            if t <= self.endTime:
                v = float(knot.GetValue())
                values.append(v)
                prevCurve = (knot.GetNextInterpolation() == Ts.InterpCurve)
                # Only include tangents if we would draw them
                if prevCurve and knot != self.knots[-1]:
                    values.append(
                        v + knot.GetPostTanWidth() * float(knot.GetPostTanSlope()))

        if not values:
            return None
        minValue = min(values)
        maxValue = max(values)
        return None if minValue == maxValue else (minValue, maxValue)

    def _updateSamples(self):
        """
        Sample the spline, should be called when attr/spline/range changes,
        since these determine sampling of the spline. This is responsible to
        populate self.polylinesToPlot with the sampled polylines.
        """
        if not self.attr:
            return
        spline = self.attr.GetSpline()
        if not spline or spline.IsEmpty():
            return
        # Make sure to 
        width = self.width() - 2 * self.margin
        height = self.height() - 2 * self.margin
        timeInterval = Gf.Interval(self.startTime, self.endTime)
        timeScale = width / (self.endTime - self.startTime)
        self.estimatedKnotMinMax = self._getEstimatedMinMaxFromSpline(spline)

        valueScale = self.DEFAULT_SAMPLING_VALUESCALE
        if self.estimatedKnotMinMax:
            valueRange = self.estimatedKnotMinMax[1] - self.estimatedKnotMinMax[0]
            valueScale = height / valueRange
        tolerance = self.DEFAULT_SAMPLING_TOLERANCE
        # TODO: use withSources and draw polyline from each source with 
        # different color?
        samples = spline.Sample(
            timeInterval, timeScale, valueScale, tolerance, withSources=False)
        if not samples:
            return
        self.polylinesToPlot = samples.polylines
        self.splinePath = QtGui.QPainterPath()
        for polyline in self.polylinesToPlot:
            # Using polyline[1:] makes a copy of the list, so use an iterator
            # to visit each item once without copying
            polyIter = iter(polyline)
            p = next(polyIter)
            self.splinePath.moveTo(p[0], p[1])
            for p in polyIter:
                self.splinePath.lineTo(p[0], p[1])

    def _transformToDevice(self):
        """
        Compute the plotting data which uses the sampled spline data, widget
        width, height, and current frame and is used for drawing.
        """
        if not self.polylinesToPlot:
            return

        ys = [pt[1] for polyline in self.polylinesToPlot for pt in polyline]
        # X is time and Y is value
        self.minX, self.maxX = self.startTime, self.endTime
        self.minY, self.maxY = min(ys), max(ys)
        if self.estimatedKnotMinMax:
            self.minY = min(self.estimatedKnotMinMax[0], self.minY)
            self.maxY = max(self.estimatedKnotMinMax[1], self.maxY)

        self.dx = max(self.maxX - self.minX, self.MIN_DELTA)
        self.dy = max(self.maxY - self.minY, self.MIN_DELTA)

        # Estimate left margin based on Y axis labels
        yLabels = [f"{self.minY + (i / self.numTicks) * self.dy:.3f}" 
            for i in range(self.numTicks + 1)]
        maxLabelWidth = max(self.fontMetrics.boundingRect(label).width() 
            for label in yLabels)
        leftMargin = maxLabelWidth + self.MARGIN_BUFFER
        self.leftMargin = leftMargin

        # Define world coordinates including all data
        worldRect = QtCore.QRectF(self.minX, self.minY, self.dx, self.dy)

        # Define view rectangle with margins
        viewRect = QtCore.QRectF(leftMargin, self.margin,
                          self.width() - leftMargin - self.margin,
                          self.height() - self.margin * 2)

        # Construct transform from world to view
        self.xform = QtGui.QTransform()
        self.xform.translate(viewRect.left(), viewRect.bottom())
        self.xform.scale(viewRect.width() / worldRect.width(),
                         -viewRect.height() / worldRect.height())
        self.xform.translate(-worldRect.left(), -worldRect.top())

    def _drawGrid(self, painter):
        gridPen = QtGui.QPen(self.GRID_COLOR)
        gridPen.setStyle(QtCore.Qt.PenStyle.DotLine)
        painter.setPen(gridPen)

        dx = self.dx / self.numTicks
        dy = self.dy / self.numTicks
        for i in range(self.numTicks):
            x = self.minX + i * dx
            p1 = self.xform.map(QtCore.QPointF(x, self.minY))
            p2 = self.xform.map(QtCore.QPointF(x, self.maxY))
            painter.drawLine(p1, p2)
            y = self.minY + i * dy
            p1 = self.xform.map(QtCore.QPointF(self.minX, y))
            p2 = self.xform.map(QtCore.QPointF(self.maxX, y))
            painter.drawLine(p1, p2)

    def _drawAxis(self, painter):
        axisPen = QtGui.QPen(self.AXIS_COLOR, 1)
        painter.setPen(axisPen)

        width = self.width()
        height = self.height()

        # X-axis
        painter.drawLine(self.leftMargin, height - self.margin, 
                         width - self.margin, height - self.margin)

        # Y-axis
        painter.drawLine(self.leftMargin, self.margin, 
                         self.leftMargin, height - self.margin)

        # --- Draw X-axis ticks and labels ---
        for i in range(self.numTicks + 1):
            t = i / self.numTicks
            xVal = self.minX + t * self.dx
            px = self.xform.map(QtCore.QPointF(xVal, 0)).x()
            painter.drawLine(px, height - self.margin, 
                             px, height - self.margin + self.AXIS_TICK_LENGTH)
            painter.drawText(
                px - self.AXIS_TICK_X_LABEL_WIDTH_OFFSET, 
                height - self.margin + self.AXIS_TICK_X_LABEL_HEIGHT_OFFSET, 
                f"{xVal:.3f}")

        # --- Draw Y-axis ticks and labels ---
        # if difference between minY and maxY is too small, only draw one tick
        # which is the minY value/maxY value itself.
        numTicksY = self.numTicks if self.dy > 1e-5 else 0
        for i in range(numTicksY + 1):
            t = i / numTicksY if numTicksY > 0 else 0
            yVal = self.minY + t * self.dy
            py = self.xform.map(QtCore.QPointF(0, yVal)).y()
            painter.drawLine(self.leftMargin - self.AXIS_TICK_LENGTH, py, 
                             self.leftMargin, py)
            painter.drawText(
                self.AXIS_TICK_Y_LABEL_WIDTH_OFFSET,
                py + self.AXIS_TICK_Y_LABEL_HEIGHT_OFFSET,
                f"{yVal:.3f}")

    def _drawKnots(self, painter):
        if not self.knots:
            return

        knotBrush = QtGui.QBrush(self.KNOT_COLOR)
        knotPen = QtGui.QPen(knotBrush, 1)

        # Offsets from the knot or tangent points to the corner of the
        # rect. kHalf is used in dual valued knots to just draw 1/2 the knot.
        kCorner = QtCore.QPointF(self.KNOT_SIZE / 2, self.KNOT_SIZE / 2)
        kHalf = QtCore.QPointF(0.0, self.KNOT_SIZE / 2)
        tCorner = QtCore.QPointF(self.TANGENT_SIZE / 2, self.TANGENT_SIZE / 2)

        painter.setPen(knotPen)
        painter.setBrush(knotBrush)

        # Enable clipping while drawing knots.  The set of knots may include
        # knots (and tangents) that are outside the bounds of the graph, but the
        # drawn knot rect and/or the knot's tangent may extend into the viewed
        # region and we want to draw the portion that's visible.
        try:
            painter.save()

            worldRect = QtCore.QRectF(self.minX, self.minY, self.dx, self.dy)
            clipRect = self.xform.mapRect(worldRect)
            painter.setClipRect(clipRect)

            # Was the previous segment a curve?
            prevCurve = False

            for knot in self.knots:
                t = knot.GetTime()
                v = float(knot.GetValue())
                valuePoint = self.xform.map(QtCore.QPointF(t, v))

                dualValued = knot.IsDualValued()
                if dualValued:
                    pv = float(knot.GetPreValue())
                    preValuePoint = self.xform.map(QtCore.QPointF(t, pv))
                else:
                    pv = v
                    preValuePoint = valuePoint

                preTanWidth = knot.GetPreTanWidth()
                preTanSlope = float(knot.GetPreTanSlope())

                postTanWidth = knot.GetPostTanWidth()
                postTanSlope = float(knot.GetPostTanSlope())

                # preTanPoint is relative to preValuePoint
                preTanPoint = self.xform.map(
                    QtCore.QPointF(t - preTanWidth,
                                   pv - (preTanWidth * preTanSlope)))

                # postTanPoint is relative to valuePoint
                postTanPoint = self.xform.map(
                    QtCore.QPointF(t + postTanWidth,
                                   v + (postTanWidth * postTanSlope)))

                # Draw tangents. Pre-tangents only draw if the previous segment
                # was a cure.
                if prevCurve:
                    painter.drawLine(preValuePoint, preTanPoint)
                    painter.fillRect(QtCore.QRectF(preTanPoint - tCorner,
                                                   preTanPoint + tCorner),
                                     knotBrush)


                # Set prevCurve for the next loop iteration, conveniently we
                # can also use it for the current iteration to draw the
                # post-tangents.
                prevCurve = (knot.GetNextInterpolation() == Ts.InterpCurve)

                # Never draw tangents past the last knot.
                if prevCurve and knot is not self.knots[-1]:
                    painter.drawLine(valuePoint, postTanPoint)
                    painter.fillRect(QtCore.QRectF(postTanPoint - tCorner,
                                                   postTanPoint + tCorner),
                                     knotBrush)
                # draw knot

                if dualValued:
                    painter.fillRect(QtCore.QRectF(preValuePoint - kCorner,
                                                   preValuePoint + kHalf),
                                     knotBrush)
                    painter.fillRect(QtCore.QRectF(valuePoint - kHalf,
                                                   valuePoint + kCorner),
                                     knotBrush)
                else:
                    painter.fillRect(QtCore.QRectF(valuePoint - kCorner,
                                                   valuePoint + kCorner),
                                     knotBrush)

        finally:
            painter.restore()

    def _drawSpline(self, painter):
        splinePen = QtGui.QPen(self.SPLINE_COLOR, 2)
        splineBrush = QtGui.QBrush(QtCore.Qt.BrushStyle.NoBrush)

        painter.setPen(splinePen)
        painter.setBrush(splineBrush)

        # Transform and draw the entire spline path at once.
        painter.drawPath(self.xform.map(self.splinePath))

    def _drawPlayheadAndCurrentValue(self, painter):
        if not self.currentFrame:
            return
        height = self.height()
        cv = self.attr.Get(self.currentFrame)
        if cv is not None:
            cv = float(cv)
        cf = self.currentFrame.GetValue()
        px = self.xform.map(QtCore.QPointF(cf, self.minY)).x()
        pen = QtGui.QPen(self.PLAYHEAD_COLOR, 2)
        painter.setPen(pen)
        painter.drawLine(px, 0, px, height)
        if cv:
            point = self.xform.map(QtCore.QPointF(cf, cv))
            radius = 4
            painter.drawEllipse(point, radius, radius)
            # draw a light hairline from this point to the y-axis
            dashedLinePen = QtGui.QPen(self.PLAYHEAD_COLOR, 1)
            dashedLinePen.setStyle(QtCore.Qt.PenStyle.DashLine)
            painter.setPen(dashedLinePen)
            painter.drawLine(point.x(), point.y(), self.leftMargin, point.y())
            # Do I need to reset the pen for the text?
            painter.setPen(pen)
            # Draw the text just above it
            text = f"{cv:.3f}"  # format the value
            font = painter.font()
            font.setPointSize(8)
            painter.setFont(font)
            textRect = painter.boundingRect(QtCore.QRectF(), text)
            textOffsetY = 6
            painter.drawText(QtCore.QPointF(
                point.x() - textRect.width() * 2, 
                point.y() - radius - textOffsetY), text)

    def resizeEvent(self, event):
        super().resizeEvent(event)
        if not self.attr:
            return
        if not self.polylinesToPlot:
            return
        self._updateSamples()
        self._transformToDevice()
        self.update()

    def paintEvent(self, event):
        if not self.attr:
            return
        if not self.polylinesToPlot:
            return

        painter = QtGui.QPainter(self)
        painter.setRenderHint(QtGui.QPainter.RenderHint.Antialiasing)

        # Draw background
        painter.fillRect(self.rect(), self.BACKGROUND_COLOR)

        # Draw grid
        self._drawGrid(painter)

        # Draw axes
        self._drawAxis(painter)

        # Draw knots
        self._drawKnots(painter)
    
        # Draw spline
        self._drawSpline(painter)

        # Draw playhead and current value
        self._drawPlayheadAndCurrentValue(painter)



#
# Copyright 2018 Pixar
#
# Licensed under the terms set forth in the LICENSE.txt file available at
# https://openusd.org/license.
#

from __future__ import print_function

import sys
import importlib

from pxr import Tf
from pxr import Plug

from .qt import QtGui, QtWidgets


class DuplicateCommandPlugin(Exception):
    """Exception raised when two command plugins are registered with the same
    name.
    """

    def __init__(self, name):
        super(DuplicateCommandPlugin, self).__init__(
            ("A command plugin with the name '{}' has already been "
            "registered.").format(name))
        self.name = name


class DeferredImport(object):
    """Defers importing a module until one of the target callable objects is
    called for the first time. Note that there is no way to know if a callable
    object exists in the target module or even if the target module exists until
    import time. All objects that are referenced are assumed to exist until
    proven otherwise when they are called (at which point an ImportError is
    raised).

    Example:

    math = DeferredImport("math")

    # You can pull as many callable objects from `math` as desired, even if they
    # don't actually exist in `math`.
    sqrt = math.sqrt
    cos = math.cos
    foo = math.foo # does not exist in the real `math` module

    # The `math` module will only be imported when this next line runs because
    # this is the first invocation of a callable object from `math`.
    cos(0)

    # This will raise an ImportError because `math.foo` doesn't really exist.
    foo(0)
    """

    def __init__(self, moduleName, packageName=None):

        self._moduleName = moduleName
        self._packageName = packageName

        self._module = None

    def __getattr__(self, attr):
        """Returns a function which calls the target function of the module and
        passes along any parameters. The module is lazy-imported when a function
        returned by this method is called for the first time.
        """

        def f(*args, **kwargs):

            if self._module is None:

                # Try to import the target module.
                try:
                    self._module = importlib.import_module(
                        self._moduleName, package=self._packageName)
                except ImportError:
                    raise ImportError(
                        "Failed deferred import: module '{}' not found.".format(
                            self._moduleName))

            # Try to get the target function from the imported module.
            try:
                moduleFunction = getattr(self._module, attr)
            except AttributeError:
                raise ImportError(("Failed deferred import: callable object "
                    " '{}' from module '{}' not found").format(
                        attr, self._moduleName))

            # Module and function loaded successfully. Now we can call the
            # function and pass it the parameters.
            return moduleFunction(*args, **kwargs)

        # Return the deferring function. It will be called at some point after
        # this method returns.
        return f


class PluginContainer(object):
    """A base class for a container which holds some Usdview plugins. Specific
    containers should inherit from this class and define the 'registerPlugins'
    and 'configureView' methods.
    """

    def deferredImport(self, moduleName):
        """Return a DeferredImport object which can be used to lazy load
        functions when they are invoked for the first time.
        """

        return DeferredImport(moduleName, self.__module__)

    def registerPlugins(self, plugRegistry, plugCtx):
        """This method is called after the container is discovered by Usdview,
        and should call 'registerCommandPlugin' one or more times on the
        plugRegistry to add commands to Usdview.
        """

        raise NotImplementedError

    def configureView(self, plugRegistry, plugUIBuilder):
        """This method is called directly after 'registerPlugins' and can be
        used to add menus which invoke a plugin command using the plugUIBuilder.
        """

        raise NotImplementedError

# We load the PluginContainer using libplug so it needs to be a defined Tf.Type.
PluginContainerTfType = Tf.Type.Define(PluginContainer)


class CommandPlugin(object):
    """A Usdview command plugin object. The plugin's `callback` parameter must
    be a callable object which takes a UsdviewApi object as its only parameter.
    """

    def __init__(self, name, displayName, callback, description, usdviewApi):

        self._name = name
        self._displayName = displayName
        self._callback = callback
        self._usdviewApi = usdviewApi
        self._description = description

    @property
    def name(self):
        """Return the command's name."""

        return self._name

    @property
    def displayName(self):
        """Return the command's display name."""

        return self._displayName

    @property
    def description(self):
        """Return the command description."""

        return self._description

    def run(self):
        """Run the command's callback function."""

        self._callback(self._usdviewApi)


class PluginMenu(object):
    """Object which adds Usdview command plugins to a QMenu."""

    def __init__(self, qMenu):

        self._qMenu = qMenu
        self._submenus = dict()

    def addItem(self, commandPlugin, shortcut=None):
        """Add a new command plugin to the menu. Optionally, provide a hotkey/
        shortcut.
        """

        action = self._qMenu.addAction(commandPlugin.displayName,
            lambda: commandPlugin.run())

        action.setToolTip(commandPlugin.description)

        if shortcut is not None:
            action.setShortcut(QtGui.QKeySequence(shortcut))

        return action

    def findOrCreateSubmenu(self, menuName):
        """Get a PluginMenu object for the submenu with the given name. If no
        submenu with the given name exists, it is created.
        """

        if menuName in self._submenus:
            return self._submenus[menuName]
        else:
            subQMenu = self._qMenu.addMenu(menuName)
            subQMenu.setToolTipsVisible(True)
            submenu = PluginMenu(subQMenu)
            self._submenus[menuName] = submenu
            return submenu

    def addSeparator(self):
        """Add a separator to the menu."""

        self._qMenu.addSeparator()


class PluginRegistry(object):
    """Manages all plugins loaded by Usdview."""

    def __init__(self, usdviewApi):

        self._usdviewApi = usdviewApi

        self._commandPlugins = dict()

    def registerCommandPlugin(self, name, displayName, callback,
            description=""):
        """Creates, registers, and returns a new command plugin.

        The plugin's `name` parameter is used to find the plugin from the
        registry later. It is good practice to prepend the plugin container's
        name to the plugin's `name` parameter to avoid duplicate names
        (i.e. "MyPluginContainer.myPluginName"). If a duplicate name is found, a
        DuplicateCommandPlugin exception will be raised.

        The `displayName` parameter is the name displayed to users.

        The plugin's `callback` parameter must be a callable object which takes
        a UsdviewApi object as its only parameter.

        The optional `description` parameter is a short description of what the
        command does which can be displayed to users.
        """

        plugin = CommandPlugin(name, displayName, callback, description,
            self._usdviewApi)
        if name in self._commandPlugins:
            raise DuplicateCommandPlugin(name)

        self._commandPlugins[name] = plugin
        return plugin

    def getCommandPlugin(self, name):
        """Finds and returns a registered command plugin. If no plugin with the
        given name is registered, return None instead.
        """

        return self._commandPlugins.get(name, None)


class PluginUIBuilder(object):
    """Used by plugins to construct UI elements in Usdview."""

    def __init__(self, mainWindow):

        self._mainWindow = mainWindow

        self._menus = dict()

        # Insert built-in menus to allow for plug-ins to add to them rather
        # than creating a new menu with duplicate title.
        for c in self._mainWindow.menuBar().children():
            if isinstance(c, QtWidgets.QMenu):
                # Strip keyboard shortcut characters so that insertion is based
                # on title alone.
                # '&'' is the only meaningful keyboard shortcut character that
                # might be present in the title. "&&" is used to insert the
                # literal '&' character but that's rare enough to not worry
                # about here.
                self._menus[str(c.title()).replace('&', '')] = PluginMenu(c)

    def findOrCreateMenu(self, menuName):
        """Get a PluginMenu object for the menu with the given name. If no menu
        with the given name exists, it is created.
        """

        if menuName in self._menus:
            return self._menus[menuName]
        else:
            qMenu = self._mainWindow.menuBar().addMenu(menuName)
            qMenu.setToolTipsVisible(True)
            menu = PluginMenu(qMenu)
            self._menus[menuName] = menu
            return menu


def loadPlugins(usdviewApi, mainWindow):
    """Find and load all Usdview plugins."""

    # Find all the defined container types using libplug.
    containerTypes = Plug.Registry.GetAllDerivedTypes(
        PluginContainerTfType)

    # Find all plugins and plugin container types through libplug.
    plugins = dict()
    for containerType in containerTypes:
        plugin = Plug.Registry().GetPluginForType(containerType)
        pluginContainerTypes = plugins.setdefault(plugin, [])
        pluginContainerTypes.append(containerType)

    # Load each plugin in alphabetical order by name. For each plugin, load all
    # of its containers in alphabetical order by type name.
    allContainers = []
    for plugin in sorted(plugins.keys(), key=lambda plugin: plugin.name):
        plugin.Load()
        pluginContainerTypes = sorted(
            plugins[plugin], key=lambda containerType: containerType.typeName)
        for containerType in pluginContainerTypes:
            if containerType.pythonClass is None:
                print(("WARNING: Missing plugin container '{}' from plugin "
                    "'{}'. Make sure the container is a defined Tf.Type and "
                    "the container's import path matches the path in "
                    "plugInfo.json.").format(
                        containerType.typeName, plugin.name), file=sys.stderr)
                continue
            container = containerType.pythonClass()
            allContainers.append(container)

    # No plugins to load, so don't create a registry.
    if len(allContainers) == 0:
        return None

    # Register all plugins from each container. If there is a naming conflict,
    # abort plugin initialization.
    registry = PluginRegistry(usdviewApi)
    for container in allContainers:
        try:
            container.registerPlugins(registry, usdviewApi)
        except DuplicateCommandPlugin as e:
            print("WARNING: {}".format(e), file=sys.stderr)
            print("Plugins will not be loaded.", file=sys.stderr)
            return None

    # Allow each plugin to construct UI elements.
    uiBuilder = PluginUIBuilder(mainWindow)
    for container in allContainers:
        container.configureView(registry, uiBuilder)

    return registry



#
# Copyright 2017 Pixar
#
# Licensed under the terms set forth in the LICENSE.txt file available at
# https://openusd.org/license.
#
def GetPySideModule():
    """Returns name of PySide module used by usdview,
    e.g. 'PySide2' or 'PySide6'"""
    # Inspect objects imported in a UI module generated by uic to determine 
    # which PySide module they come from (e.g. PySide2.QtCore).
    # This insulates the code from assuming that the generated code will
    # import something specific.
    from . import attributeValueEditorUI
    import inspect

    for name in dir(attributeValueEditorUI):
        obj = getattr(attributeValueEditorUI, name)
        module = inspect.getmodule(obj)
        if module and module.__name__.startswith('PySide'):
            return module.__name__.split('.')[0]

    return None
        
PySideModule = GetPySideModule()
if PySideModule == 'PySide2':
    from PySide2 import QtCore, QtGui, QtWidgets, QtOpenGL
    from PySide2.QtOpenGL import QGLWidget as QGLWidget
    from PySide2.QtOpenGL import QGLFormat as QGLFormat
    from PySide2 import QtWidgets as QtActionWidgets
    
    # Older versions still have QtGui.QStringListModel - this
    # is apparently a bug:
    #    https://bugreports.qt.io/browse/PYSIDE-614
    if not hasattr(QtCore, 'QStringListModel'):
        QtCore.QStringListModel = QtGui.QStringListModel

    def isContextInitialised(self):
        return self.context().initialized()
    
    QGLWidget.isContextInitialised = isContextInitialised

    def bindTexture(self, qimage):
        from OpenGL import GL
        tex = self.bindTexture(qimage, GL.GL_TEXTURE_2D, GL.GL_RGBA,
                               QtOpenGL.QGLContext.NoBindOption)
        GL.glBindTexture(GL.GL_TEXTURE_2D, tex)

        return tex

    def releaseTexture(self, tex):
        from OpenGL import GL
        GL.glDeleteTextures(tex)

    QGLWidget.BindTexture = bindTexture
    QGLWidget.ReleaseTexture = releaseTexture

    def initQGLWidget(self, glFormat, parent):
        QGLWidget.__init__(self, glFormat, parent)

    QGLWidget.InitQGLWidget = initQGLWidget

elif PySideModule == 'PySide6':
    from PySide6 import QtCore, QtGui, QtWidgets, QtOpenGL
    from PySide6.QtOpenGLWidgets import QOpenGLWidget as QGLWidget
    from PySide6.QtGui import QSurfaceFormat as QGLFormat
    from PySide6 import QtGui as QtActionWidgets

    def isContextInitialised(self):
        return True

    QGLWidget.isContextInitialised = isContextInitialised

    QGLWidget.updateGL = QGLWidget.update

    if not hasattr(QGLWidget, 'grabFrameBuffer'):
        QGLWidget.grabFrameBuffer = QGLWidget.grabFramebuffer

    def bindTexture(self, qimage):
        tex = QtOpenGL.QOpenGLTexture(qimage)
        tex.bind()
        return tex

    def releaseTexture(self, tex):
        tex.release()
        tex.destroy()

    QGLWidget.BindTexture = bindTexture
    QGLWidget.ReleaseTexture = releaseTexture

    if not hasattr(QGLFormat, 'setSampleBuffers'):
        QGLFormat.setSampleBuffers = lambda self, _: None

    def initQGLWidget(self, glFormat, parent):
        QGLWidget.__init__(self)
        self.setFormat(glFormat)

    QGLWidget.InitQGLWidget = initQGLWidget

else:
    raise ImportError('Unrecognized PySide module "{}"'.format(PySideModule))



#
# Copyright 2016 Pixar
#
# Licensed under the terms set forth in the LICENSE.txt file available at
# https://openusd.org/license.
#
from __future__ import print_function

from .qt import QtCore, QtGui, QtWidgets
from pxr import Sdf, Usd, UsdGeom
from pxr.UsdUtils.constantsGroup import ConstantsGroup
from ._usdviewq import Utils

from .common import UIPrimTypeColors, UIFonts

HALF_DARKER = 150

# Pulled out as a wrapper to facilitate cprofile tracking
def _GetPrimInfo(prim, time):
    return Utils.GetPrimInfo(prim, time)


class PrimViewColumnIndex(ConstantsGroup):
    NAME, TYPE, VIS, GUIDES, DRAWMODE = range(5)

# This class extends QTreeWidgetItem to also contain all the stage
# prim data associated with it and populate itself with that data.

class PrimViewItem(QtWidgets.QTreeWidgetItem):
    def __init__(self, prim, appController, primHasChildren):
        # Do *not* pass a parent.  The client must build the hierarchy.
        # This can dramatically improve performance when building a
        # large hierarchy.
        super(PrimViewItem, self).__init__()

        self.prim = prim
        self._appController = appController
        self._needsPull = True
        self._needsPush = True
        self._needsChildrenPopulated = primHasChildren

        # Initialize these now so loadVis(), _visData() and onClick() can
        # use them without worrying if _pull() has been called.
        self.imageable = False
        self.active = False
        self.vis = False

        # True if this item is an ancestor of a selected item.
        self.ancestorOfSelected = False

        # If we know we'll have children show a norgie, otherwise don't.
        if primHasChildren:
            self.setChildIndicatorPolicy(
                    QtWidgets.QTreeWidgetItem.ChildIndicatorPolicy.ShowIndicator)
        else:
            self.setChildIndicatorPolicy(
                    QtWidgets.QTreeWidgetItem.ChildIndicatorPolicy.DontShowIndicator)

        # If this item includes a persistent drawMode widget, it is stored here.
        self.drawModeWidget = None
        
    def push(self):
        """Pushes prim data to the UI."""
        # Push to UI.
        if self._needsPush:
            self._needsPush = False
            self._pull()

    def _pull(self):
        """Extracts and stores prim data."""
        if self._needsPull:
            # Only do this once.
            self._needsPull = False

            # Visibility is recursive so the parent must pull before us.
            parent = self.parent()
            if isinstance(parent, PrimViewItem):
                parent._pull()

            # Get our prim info.
            # To avoid Python overhead, request data in batch from C++.
            info = _GetPrimInfo(
                self.prim, self._appController._dataModel.currentFrame)
            self._extractInfo(info)

            self.emitDataChanged()

    @staticmethod
    def _HasAuthoredDrawMode(prim):
        modelAPI = UsdGeom.ModelAPI(prim)
        drawModeAttr = modelAPI.GetModelDrawModeAttr()
        return drawModeAttr and drawModeAttr.HasAuthoredValue()

    def _isComputedDrawModeInherited(self, parentDrawModeIsInherited=None):
        """Returns true if the computed draw mode for this item is inherited 
           from an authored "model:drawMode" value on an ancestor prim.
        """
        if PrimViewItem._HasAuthoredDrawMode(self.prim):
            return False

        parent = self.prim.GetParent()
        while parent and parent.GetPath() != Sdf.Path.absoluteRootPath:
            if PrimViewItem._HasAuthoredDrawMode(parent):
                return True

            # Stop the upward traversal if we know whether the parent's draw 
            # mode is inherited.
            if parentDrawModeIsInherited is not None:
                return parentDrawModeIsInherited
            
            parent = parent.GetParent()
        return False

    def _extractInfo(self, info):
        ( self.hasArcs,
          self.active,
          self.imageable,
          self.defined,
          self.abstract,
          self.isInPrototype,
          self.isInstance,
          self.supportsGuides,
          self.supportsDrawMode,
          isVisibilityInherited,
          self.visVaries,
          self.name,
          self.typeName,
          self.displayName ) = info

        parent = self.parent()
        parentIsPrimViewItem = isinstance(parent, PrimViewItem)
        self.computedVis =  parent.computedVis if parentIsPrimViewItem \
                            else UsdGeom.Tokens.inherited
        if self.imageable and self.active:
            if isVisibilityInherited:
                self.vis = UsdGeom.Tokens.inherited
            else:
                self.vis = self.computedVis = UsdGeom.Tokens.invisible

        # If this is the invisible root item, initialize fallback values for 
        # the drawMode related parameters.
        if not parentIsPrimViewItem:
            self.computedDrawMode = ''
            self.isDrawModeInherited = False
            return

        # We don't need to compute drawMode related parameters for primViewItems
        # that don't support draw mode.
        if not self.supportsDrawMode:
            return
        
        self.computedDrawMode = UsdGeom.ModelAPI(self.prim).ComputeModelDrawMode(
            parent.computedDrawMode) if parentIsPrimViewItem else ''


        parentDrawModeIsInherited = parent.isDrawModeInherited if \
                parentIsPrimViewItem else None
        
        self.isDrawModeInherited = self._isComputedDrawModeInherited(
                    parentDrawModeIsInherited)
        

    def addChildren(self, children):
        """Adds children to the end of this item.  This is the only
           method clients should call to manage an item's children."""
        self._needsChildrenPopulated = False
        super(PrimViewItem, self).addChildren(children)

    def data(self, column, role):
        # All Qt queries that affect the display of this item (label, font,
        # color, etc) will come through this method.  We set that data
        # lazily because it's expensive to do and (for the most part) Qt
        # only needs the data for items that are visible.
        self.push()

        # We report data directly rather than use set...() and letting
        # super().data() return it because Qt can be pathological during
        # calls to set...() when the item hierarchy is attached to a view,
        # making thousands of calls to data().
        result = None
        if column == PrimViewColumnIndex.NAME:
            result = self._nameData(role)
        elif column == PrimViewColumnIndex.TYPE:
            result = self._typeData(role)
        elif column == PrimViewColumnIndex.VIS:
            result = self._visData(role)
        elif column == PrimViewColumnIndex.GUIDES and self.supportsGuides:
            # XXX Temp fix to prevent API calls from throwing an exception since
            # this method is being called on shutdown, after the stage has
            # closed, on expired prims
            if self.prim:
                result = self._guideData(role)
        elif column == PrimViewColumnIndex.DRAWMODE and self.supportsDrawMode:
            result = self._drawModeData(role)
        if not result:
            result = super(PrimViewItem, self).data(column, role)
        return result

    def _GetForegroundColor(self):
        self.push()
        
        if self.isInstance:
            color = UIPrimTypeColors.INSTANCE
        elif self.hasArcs:
            color = UIPrimTypeColors.HAS_ARCS
        elif self.isInPrototype:
            color = UIPrimTypeColors.PROTOTYPE
        else:
            color = UIPrimTypeColors.NORMAL
        return color.color() if self.active else color.color().darker(HALF_DARKER)

    def _nameData(self, role):
        if role == QtCore.Qt.ItemDataRole.DisplayRole:
            if self._appController._dataModel.viewSettings.showPrimDisplayNames:
                return self.displayName if self.displayName else self.name
            else:
                return self.name
        elif role == QtCore.Qt.ItemDataRole.FontRole:
            # Abstract prims are also considered defined; since we want
            # to distinguish abstract defined prims from non-abstract
            # defined prims, we check for abstract first.
            if self.abstract:
                return UIFonts.ABSTRACT_PRIM
            elif not self.defined:
                return UIFonts.OVER_PRIM
            else:
                return UIFonts.DEFINED_PRIM
        elif role == QtCore.Qt.ItemDataRole.ForegroundRole:
            return self._GetForegroundColor()
        elif role == QtCore.Qt.ItemDataRole.ToolTipRole:
            toolTip = 'Prim'
            if len(self.typeName) > 0:
                toolTip = self.typeName + ' ' + toolTip
            if self.isInPrototype:
                toolTip = 'Prototype ' + toolTip
            if not self.defined:
                toolTip = 'Undefined ' + toolTip
            elif self.abstract:
                toolTip = 'Abstract ' + toolTip
            else:
                toolTip = 'Defined ' + toolTip
            if not self.active:
                toolTip = 'Inactive ' + toolTip
            elif self.isInstance:
                toolTip = 'Instanced ' + toolTip
        
            # tooltip should always show both name and display name
            toolTip = toolTip + "<br>Name: " + self.name
            if self.displayName:
                toolTip = toolTip + "<br>Display Name: " + self.displayName

            if self.hasArcs:
                toolTip = toolTip + "<br>Has composition arcs"
            return toolTip
        else:
            return None

    def _drawModeData(self, role):
        if role == QtCore.Qt.ItemDataRole.DisplayRole:
            return self.computedDrawMode
        elif role == QtCore.Qt.ItemDataRole.FontRole:
            return UIFonts.BOLD_ITALIC if self.isDrawModeInherited else \
                   UIFonts.DEFINED_PRIM
        elif role == QtCore.Qt.ItemDataRole.ForegroundRole:
            color = self._GetForegroundColor()
            return color.darker(110) if self.isDrawModeInherited else \
                   color

    def _typeData(self, role):
        if role == QtCore.Qt.ItemDataRole.DisplayRole:
            return self.typeName
        else:
            return self._nameData(role)
            
    def _isVisInherited(self):
        return self.imageable and self.active and \
               self.vis != UsdGeom.Tokens.invisible and \
               self.computedVis == UsdGeom.Tokens.invisible

    def _visData(self, role):
        if role == QtCore.Qt.ItemDataRole.DisplayRole:
            if self.imageable and self.active:
                return "I" if self.vis == UsdGeom.Tokens.invisible else "V"
            else:
                return ""
        elif role == QtCore.Qt.ItemDataRole.TextAlignmentRole:
            return QtCore.Qt.AlignmentFlag.AlignCenter
        elif role == QtCore.Qt.ItemDataRole.FontRole:
            return UIFonts.BOLD_ITALIC if self._isVisInherited() \
                   else UIFonts.BOLD
        elif role == QtCore.Qt.ItemDataRole.ForegroundRole:
            fgColor = self._GetForegroundColor()
            return fgColor.darker() if self._isVisInherited() \
                   else fgColor
        elif role == QtCore.Qt.ItemDataRole.ToolTipRole:
            if self.imageable and self.active:
                if self.vis == UsdGeom.Tokens.invisible:
                    return "Invisible Prim"
                else:
                    return "Visible Prim"
        else:
            return None

    def _guideData(self, role):
        if role == QtCore.Qt.ItemDataRole.DisplayRole:
            if (UsdGeom.VisibilityAPI(self.prim).GetGuideVisibilityAttr().Get()
                == UsdGeom.Tokens.visible):
                return "V"
            else:
                return "I"
        elif role == QtCore.Qt.ItemDataRole.TextAlignmentRole:
            return QtCore.Qt.AlignmentFlag.AlignCenter
        elif role == QtCore.Qt.ItemDataRole.FontRole:
            if self._isVisInherited() or self.vis == UsdGeom.Tokens.invisible:
                return UIFonts.BOLD_ITALIC
            else:
                return UIFonts.BOLD
        elif role == QtCore.Qt.ItemDataRole.ForegroundRole:
            fgColor = self._GetForegroundColor()
            if self._isVisInherited() or self.vis == UsdGeom.Tokens.invisible:
                return fgColor.darker()
            else:
                return fgColor
        elif role == QtCore.Qt.ItemDataRole.ToolTipRole:
            if (UsdGeom.VisibilityAPI(self.prim).GetGuideVisibilityAttr().Get()
                == UsdGeom.Tokens.visible):
                return "Visible Guides"
            else:
                return "Invisible Guides"
        else:
            return None

    def needsChildrenPopulated(self):
        return self._needsChildrenPopulated

    def canChangeVis(self):
        if not self.imageable:
            print("WARNING: The prim <" + str(self.prim.GetPath()) + \
                    "> is not imageable. Cannot change visibility.")
            return False
        elif self.isInPrototype:
            print("WARNING: The prim <" + str(self.prim.GetPath()) + \
                   "> is in a prototype. Cannot change visibility.")
            return False
        return True

    def loadVis(self, inheritedVis, visHasBeenAuthored):
        if not (self.imageable and self.active):
            return inheritedVis

        time = self._appController._dataModel.currentFrame
        # If visibility-properties have changed on the stage, then
        # we must re-evaluate our variability before deciding whether
        # we can avoid re-reading our visibility
        visAttr = UsdGeom.Imageable(self.prim).GetVisibilityAttr()
        if visHasBeenAuthored:
            self.visVaries = visAttr.ValueMightBeTimeVarying()

            if not self.visVaries:
                self.vis = visAttr.Get(time)

        if self.visVaries:
            self.vis =  visAttr.Get(time)

        self.computedVis = UsdGeom.Tokens.invisible \
            if self.vis == UsdGeom.Tokens.invisible \
            else inheritedVis

        self.emitDataChanged()
        return self.computedVis

    @staticmethod
    def propagateDrawMode(item, primView, parentDrawMode='', 
                          parentDrawModeIsInherited=None):
        # If this item does not support draw mode, none of its descendants 
        # can support it. Hence, stop recursion here.
        # 
        # Call push() here to ensure that supportsDrawMode has been populated
        # for the item.
        item.push()
        if not item.supportsDrawMode:
            return

        from .primTreeWidget import DrawModeWidget
        drawModeWidget = item.drawModeWidget
        if drawModeWidget:
            drawModeWidget.RefreshDrawMode()
        else:
            modelAPI = UsdGeom.ModelAPI(item.prim)
            item.computedDrawMode = modelAPI.ComputeModelDrawMode(parentDrawMode)
            item.isDrawModeInherited = item._isComputedDrawModeInherited(
                    parentDrawModeIsInherited=parentDrawModeIsInherited)
            item.emitDataChanged()

        # Traverse down to children to update their drawMode.
        for child in [item.child(i) for i in range(item.childCount())]:
            PrimViewItem.propagateDrawMode(child, primView, 
                    parentDrawMode=item.computedDrawMode,
                    parentDrawModeIsInherited=item.isDrawModeInherited)

    @staticmethod
    def propagateVis(item, authoredVisHasChanged=True):
        parent = item.parent()
        inheritedVis = parent._resetAncestorsRecursive(authoredVisHasChanged) \
            if isinstance(parent, PrimViewItem) \
            else UsdGeom.Tokens.inherited
        # This may be called on the "InvisibleRootItem" that is an ordinary
        # QTreeWidgetItem, in which case we need to process its children
        # individually
        if isinstance(item, PrimViewItem):
            item._pushVisRecursive(inheritedVis, authoredVisHasChanged)
        else:
            for child in [item.child(i) for i in range(item.childCount())]:
                child._pushVisRecursive(inheritedVis, authoredVisHasChanged)

    def _resetAncestorsRecursive(self, authoredVisHasChanged):
        parent = self.parent()
        inheritedVis = parent._resetAncestorsRecursive(authoredVisHasChanged) \
            if isinstance(parent, PrimViewItem) \
            else UsdGeom.Tokens.inherited

        return self.loadVis(inheritedVis, authoredVisHasChanged)

    def _pushVisRecursive(self, inheritedVis, authoredVisHasChanged):
        myComputedVis = self.loadVis(inheritedVis, authoredVisHasChanged)

        for child in [self.child(i) for i in range(self.childCount())]:
            child._pushVisRecursive(myComputedVis, authoredVisHasChanged)

    def setLoaded(self, loaded):
        if self.prim.IsPrototype():
            print("WARNING: The prim <" + str(self.prim.GetPath()) + \
                   "> is a prototype prim. Cannot change load state.")
            return

        if self.prim.IsActive():
            if loaded:
                self.prim.Load()
            else:
                self.prim.Unload()

    def setVisible(self, visible):
        if self.canChangeVis():
            UsdGeom.Imageable(self.prim).GetVisibilityAttr().Set(UsdGeom.Tokens.inherited
                             if visible else UsdGeom.Tokens.invisible)
            self.visChanged()

    def makeVisible(self):
        if self.canChangeVis():
            # It is in general not kosher to use an Sdf.ChangeBlock around
            # operations at the Usd API level.  We have carefully arranged
            # (with insider knowledge) to have only "safe" mutations
            # happening inside the ChangeBlock.  We do this because
            # UsdImaging updates itself independently for each
            # Usd.Notice.ObjectsChanged it receives.  We hope to eliminate
            # the performance need for this by addressing bug #121992
            from pxr import Sdf
            with Sdf.ChangeBlock():
                UsdGeom.Imageable(self.prim).MakeVisible()
            self.visChanged()

    def visChanged(self):
        # called when user authors a new visibility value
        # we must re-determine if visibility is varying over time
        self.loadVis(self.parent().computedVis, True)

    def toggleVis(self):
        """Return True if the the prim's visibility state was toggled. """
        if self.imageable and self.active:
            self.setVisible(self.vis == UsdGeom.Tokens.invisible)
            PrimViewItem.propagateVis(self)
            return True
        return False

    def toggleGuides(self):
        """Return True if the the prim's guide visibility state was toggled."""
        if not self.supportsGuides:
            return False
        attr = (
            UsdGeom.VisibilityAPI.Apply(self.prim).CreateGuideVisibilityAttr())
        return attr.Set(UsdGeom.Tokens.invisible
            if attr.Get() == UsdGeom.Tokens.visible else UsdGeom.Tokens.visible)



#
# Copyright 2016 Pixar
#
# Licensed under the terms set forth in the LICENSE.txt file available at
# https://openusd.org/license.
#
from pxr import Tf

Tf.ScriptModuleLoader()._LoadModulesForLibrary('LoadsAll')

# Request to load everything.  This will be deferred by the TfScriptModuleLoader
# until after the current dependency load is complete.
Tf.ScriptModuleLoader()._LoadModulesForLibrary('')



#
# Copyright 2024 Pixar
#
# Licensed under the terms set forth in the LICENSE.txt file available at
# https://openusd.org/license.
#

from pxr import Tf
Tf.PreparePythonModule()
del Tf

# Test framework isn't always built.
try:
    from .TsTest_Baseliner import TsTest_Baseliner
    from .TsTest_Grapher import TsTest_Grapher
    from .TsTest_Comparator import TsTest_Comparator
except ImportError:
    pass



#
# Copyright 2016 Pixar
#
# Licensed under the terms set forth in the LICENSE.txt file available at
# https://openusd.org/license.
#
from pxr import Usd
from .qt import QtCore, QtWidgets
from .attributeValueEditorUI import Ui_AttributeValueEditor
from .common import GetPropertyColor, UIPropertyValueSourceColors
from .scalarTypes import ToString

# This is the widget that appears when selecting an attribute and
# opening the "Value" tab.

class AttributeValueEditor(QtWidgets.QWidget):
    editComplete = QtCore.Signal('QString')

    def __init__(self, parent):
        QtWidgets.QWidget.__init__(self, parent)
        self._ui = Ui_AttributeValueEditor()
        self._ui.setupUi(self)

        self._defaultView = self._ui.valueViewer

        from .arrayAttributeView import ArrayAttributeView
        from .splineViewer import SplineViewer
        self._extraAttrViews = [
                ArrayAttributeView(self),
                SplineViewer(parent=self),
                ]

        for attrView in self._extraAttrViews:
            self._ui.stackedWidget.addWidget(attrView)

        self.clear()

    def setAppController(self, appController):
        # pass the appController instance from which to retrieve
        # variable data.
        self._appController = appController
        for attrView in self._extraAttrViews:
            self._appController._dataModel.selection \
                .signalPrimSelectionChanged.connect(self.clear)
            from .splineViewer import SplineViewer
            if isinstance(attrView, SplineViewer):
                self._appController._dataModel.currentFrameChanged.connect(
                    attrView.SetCurrentFrame)
                self._appController._dataModel.frameRangeChanged.connect(
                    attrView.SetStartAndEndTime)

    def populate(self, primPath, propName):
        # called when the selected attribute has changed
        self._primPath = primPath

        try:
            self._attribute = self._appController._propertiesDict[propName]
        except KeyError:
            self._attribute = None

        self._isSet = True  # an attribute is selected

        self.refresh()  # load the value at the current frame

    def _FindView(self, attr):
        # Early-out for CustomAttributes and Relationships
        if not isinstance(attr, Usd.Attribute):
            return None

        for attrView in self._extraAttrViews:
            if attrView.CanView(attr):
                return attrView

        return None

    def refresh(self):
        # usually called upon frame change or selected attribute change
        if not self._isSet:
            return

        # attribute connections and relationship targets have no value to display
        # in the value viewer.
        if self._attribute is None:
            return

        # If the current attribute doesn't belong to the current prim, don't
        # display its value.
        if self._attribute.GetPrimPath() != self._primPath:
            if whichView := self._FindView(self._attribute):
                whichView.Clear()
            self._ui.valueViewer.setText("")
            return

        frame = self._appController._dataModel.currentFrame

        # get the value of the attribute
        if isinstance(self._attribute, Usd.Relationship):
            self._val = self._attribute.GetTargets()
        else: # Usd.Attribute or CustomAttribute
            self._val = self._attribute.Get(frame)

        whichView = self._FindView(self._attribute)
        if whichView:
            self._ui.stackedWidget.setCurrentWidget(whichView)
            whichView.SetAttribute(self._attribute, frame)
        else:
            self._ui.stackedWidget.setCurrentWidget(self._defaultView)
            txtColor = GetPropertyColor(self._attribute, frame)

            # set text and color in the value viewer
            self._ui.valueViewer.setTextColor(txtColor)

            if isinstance(self._attribute, Usd.Relationship):
                typeName = None
            else:
                typeName = self._attribute.GetTypeName()
            rowText = ToString(self._val, typeName)
            self._ui.valueViewer.setText(rowText)

    def clear(self):
        # set the value editor to 'no attribute selected' mode
        self._isSet = False
        self._ui.valueViewer.setText("")
        for attrView in self._extraAttrViews:
            attrView.Clear()
        # make sure we're showing the default view
        self._ui.stackedWidget.setCurrentWidget(self._defaultView)



#
# Copyright 2018 Pixar
#
# Licensed under the terms set forth in the LICENSE.txt file available at
# https://openusd.org/license.
#

from .qt import QtCore
from pxr import UsdGeom, Sdf
from pxr.UsdAppUtils.complexityArgs import RefinementComplexities

from .common import (RenderModes, ColorCorrectionModes, PickModes, 
                     SelectionHighlightModes, CameraMaskModes, 
                     PrintWarning)

from . import settings
from .settings import StateSource
from pxr.UsdUtils.constantsGroup import ConstantsGroup
from .freeCamera import FreeCamera
from .common import ClearColors, HighlightColors


# Map of clear color names to rgba color tuples in linear space.
_CLEAR_COLORS_DICT = {
    ClearColors.BLACK:       (0.0, 0.0, 0.0, 1.0),
    ClearColors.DARK_GREY:   (0.07074, 0.07074, 0.07074, 1.0),
    ClearColors.LIGHT_GREY:  (0.45626, 0.45626, 0.45626, 1.0),
    ClearColors.WHITE:       (1.0, 1.0, 1.0, 1.0)}


# Map of highlight color names to rgba color tuples in linear space.
_HIGHLIGHT_COLORS_DICT = {
    HighlightColors.WHITE:   (1.0, 1.0, 1.0, 0.5),
    HighlightColors.YELLOW:  (1.0, 1.0, 0.0, 0.5),
    HighlightColors.CYAN:    (0.0, 1.0, 1.0, 0.5)}


# Default values for default material components.
DEFAULT_AMBIENT = 0.2
DEFAULT_SPECULAR = 0.1


def visibleViewSetting(f):
    def wrapper(self, *args, **kwargs):
        f(self, *args, **kwargs)
        # If f raises an exception, the signal is not emitted.
        self.signalVisibleSettingChanged.emit()
        self.signalSettingChanged.emit()
    return wrapper


def invisibleViewSetting(f):
    def wrapper(self, *args, **kwargs):
        f(self, *args, **kwargs)
        # If f raises an exception, the signal is not emitted.
        self.signalSettingChanged.emit()
    return wrapper


def freeCameraViewSetting(f):
    def wrapper(self, *args, **kwargs):
        f(self, *args, **kwargs)
        # If f raises an exception, the signal is not emitted.
        self.signalFreeCameraSettingChanged.emit()
        self.signalSettingChanged.emit()
    return wrapper


class OCIOSettings():
    """Class to hold OCIO display, view, and colorSpace config settings
    as strings."""

    def __init__(self, display="", view="", colorSpace=""):
        self._display = display
        self._view = view
        self._colorSpace = colorSpace

    @property
    def display(self):
        return self._display

    @property
    def view(self):
        return self._view

    @property
    def colorSpace(self):
        return self._colorSpace

class ViewSettingsDataModel(StateSource, QtCore.QObject):
    """Data model containing settings related to the rendered view of a USD
    file.
    """

    # emitted when any view setting changes
    signalSettingChanged = QtCore.Signal()

    # emitted when any view setting which may affect the rendered image changes
    signalVisibleSettingChanged = QtCore.Signal()

    # emitted when any view setting that affects the free camera changes. This
    # signal allows clients to switch to the free camera whenever its settings
    # are modified. Some operations may cause this signal to be emitted multiple
    # times.
    signalFreeCameraSettingChanged = QtCore.Signal()

    # emitted when autoClipping changes value, so that clients can initialize
    # it efficiently.  This signal will be emitted *before* 
    # signalVisibleSettingChanged when autoClipping changes.
    signalAutoComputeClippingPlanesChanged = QtCore.Signal()

    # emitted when any aspect of the defaultMaterial changes
    signalDefaultMaterialChanged = QtCore.Signal()

    # emitted when any setting affecting the GUI style changes
    signalStyleSettingsChanged = QtCore.Signal()

    def __init__(self, rootDataModel, parent):
        QtCore.QObject.__init__(self)
        StateSource.__init__(self, parent, "model")

        self._rootDataModel = rootDataModel

        self._cameraMaskColor = tuple(self.stateProperty("cameraMaskColor", default=[0.1, 0.1, 0.1, 1.0]))
        self._cameraReticlesColor = tuple(self.stateProperty("cameraReticlesColor", default=[0.0, 0.7, 1.0, 1.0]))
        self._defaultMaterialAmbient = self.stateProperty("defaultMaterialAmbient", default=DEFAULT_AMBIENT)
        self._defaultMaterialSpecular = self.stateProperty("defaultMaterialSpecular", default=DEFAULT_SPECULAR)
        self._redrawOnScrub = self.stateProperty("redrawOnScrub", default=True)
        self._stepSize = self.stateProperty("stepSize", default=1.0)
        self._renderMode = self.stateProperty("renderMode", default=RenderModes.SMOOTH_SHADED)
        self._freeCameraFOV = self.stateProperty("freeCameraFOV", default=60.0)
        self._freeCameraAspect = self.stateProperty("freeCameraAspect", default=1.0)
        # For freeCameraOverrideNear/Far, Use -inf as a sentinel value to mean
        # None. (We cannot use None directly because that would cause a type-
        # checking error in Settings.)
        self._clippingPlaneNoneValue = float('-inf')
        self._freeCameraOverrideNear = self.stateProperty("freeCameraOverrideNear", default=self._clippingPlaneNoneValue)
        self._freeCameraOverrideFar = self.stateProperty("freeCameraOverrideFar", default=self._clippingPlaneNoneValue)
        if self._freeCameraOverrideNear == self._clippingPlaneNoneValue:
            self._freeCameraOverrideNear = None
        if self._freeCameraOverrideFar == self._clippingPlaneNoneValue:
            self._freeCameraOverrideFar = None
        self._lockFreeCameraAspect = self.stateProperty("lockFreeCameraAspect", default=False)
        self._colorCorrectionMode = self.stateProperty("colorCorrectionMode", default=ColorCorrectionModes.SRGB)
        self._ocioSettings = OCIOSettings()
        self._pickMode = self.stateProperty("pickMode", default=PickModes.PRIMS)

        # We need to store the trinary selHighlightMode state here,
        # because the stageView only deals in True/False (because it
        # cannot know anything about playback state).
        self._selHighlightMode = self.stateProperty("selectionHighlightMode", default=SelectionHighlightModes.ONLY_WHEN_PAUSED)

        # We store the highlightColorName so that we can compare state during
        # initialization without inverting the name->value logic
        self._highlightColorName = self.stateProperty("highlightColor", default="Yellow")
        self._ambientLightOnly = self.stateProperty("cameraLightEnabled", default=True)
        self._domeLightEnabled = self.stateProperty("domeLightEnabled", default=False)
        self._domeLightTexturesVisible = self.stateProperty("domeLightTexturesVisible", default=True)
        self._clearColorText = self.stateProperty("backgroundColor", default="Grey (Dark)")
        self._autoComputeClippingPlanes = self.stateProperty("autoComputeClippingPlanes", default=False)
        self._showBBoxPlayback = self.stateProperty("showBBoxesDuringPlayback", default=False)
        self._showBBoxes = self.stateProperty("showBBoxes", default=True)
        self._showAABBox = self.stateProperty("showAABBox", default=True)
        self._showOBBox = self.stateProperty("showOBBox", default=True)
        self._displayGuide = self.stateProperty("displayGuide", default=False)
        self._displayProxy = self.stateProperty("displayProxy", default=True)
        self._displayRender = self.stateProperty("displayRender", default=False)
        self._enableSceneMaterials = self.stateProperty("enableSceneMaterials", default=True)
        self._enableSceneLights = self.stateProperty("enableSceneLights", default=True)
        self._cullBackfaces = self.stateProperty("cullBackfaces", default=False)
        self._showInactivePrims = self.stateProperty("showInactivePrims", default=True)

        showAllMasterPrims = self.stateProperty("showAllMasterPrims", default=False)
        self._showAllPrototypePrims = self.stateProperty("showAllPrototypePrims", default=False)
        # XXX: For backwards compatibility, we use the "showAllMasterPrims"
        # saved state to drive the new "showAllPrototypePrims" state. We
        # can remove "showAllMasterPrims" in a later release.
        self._showAllPrototypePrims = showAllMasterPrims

        self._showUndefinedPrims = self.stateProperty("showUndefinedPrims", default=False)
        self._showAbstractPrims = self.stateProperty("showAbstractPrims", default=False)
        self._showPrimDisplayNames = self.stateProperty("showPrimDisplayNames", default=True)
        self._rolloverPrimInfo = self.stateProperty("rolloverPrimInfo", default=False)
        self._displayCameraOracles = self.stateProperty("cameraOracles", default=False)
        self._cameraMaskMode = self.stateProperty("cameraMaskMode", default=CameraMaskModes.NONE)
        self._showMask_Outline = self.stateProperty("cameraMaskOutline", default=False)
        self._showReticles_Inside = self.stateProperty("cameraReticlesInside", default=False)
        self._showReticles_Outside = self.stateProperty("cameraReticlesOutside", default=False)
        self._showHUD = self.stateProperty("showHUD", default=True)

        self._showHUD_Info = self.stateProperty("showHUDInfo", default=False)
        # XXX Until we can make the "Subtree Info" stats-gathering faster
        # we do not want the setting to persist from session to session.
        self._showHUD_Info = False

        self._showHUD_Complexity = self.stateProperty("showHUDComplexity", default=True)
        self._showHUD_Performance = self.stateProperty("showHUDPerformance", default=True)
        self._showHUD_GPUstats = self.stateProperty("showHUDGPUStats", default=False)

        self._complexity = RefinementComplexities.LOW
        self._freeCamera = None
        self._cameraPath = None
        self._fontSize = self.stateProperty("fontSize", default=10)
        self._appIconMode = self.stateProperty("appIconMode", default="Automatic")

    def onSaveState(self, state):
        state["cameraMaskColor"] = list(self._cameraMaskColor)
        state["cameraReticlesColor"] = list(self._cameraReticlesColor)
        state["defaultMaterialAmbient"] = self._defaultMaterialAmbient
        state["defaultMaterialSpecular"] = self._defaultMaterialSpecular
        state["redrawOnScrub"] = self._redrawOnScrub
        state["stepSize"] = self._stepSize
        state["renderMode"] = self._renderMode
        state["freeCameraFOV"] = self._freeCameraFOV
        freeCameraOverrideNear = self._freeCameraOverrideNear
        if freeCameraOverrideNear is None:
            freeCameraOverrideNear = self._clippingPlaneNoneValue
        state["freeCameraOverrideNear"] = freeCameraOverrideNear
        freeCameraOverrideFar = self._freeCameraOverrideFar
        if freeCameraOverrideFar is None:
            freeCameraOverrideFar = self._clippingPlaneNoneValue
        state["freeCameraOverrideFar"] = freeCameraOverrideFar
        state["freeCameraAspect"] = self._freeCameraAspect
        state["lockFreeCameraAspect"] = self._lockFreeCameraAspect
        state["colorCorrectionMode"] = self._colorCorrectionMode
        state["pickMode"] = self._pickMode
        state["selectionHighlightMode"] = self._selHighlightMode
        state["highlightColor"] = self._highlightColorName
        state["cameraLightEnabled"] = self._ambientLightOnly
        state["domeLightEnabled"] = self._domeLightEnabled
        state["domeLightTexturesVisible"] = self._domeLightTexturesVisible
        state["backgroundColor"] = self._clearColorText
        state["autoComputeClippingPlanes"] = self._autoComputeClippingPlanes
        state["showBBoxesDuringPlayback"] = self._showBBoxPlayback
        state["showBBoxes"] = self._showBBoxes
        state["showAABBox"] = self._showAABBox
        state["showOBBox"] = self._showOBBox
        state["displayGuide"] = self._displayGuide
        state["displayProxy"] = self._displayProxy
        state["displayRender"] = self._displayRender
        state["enableSceneMaterials"] = self._enableSceneMaterials
        state["enableSceneLights"] = self._enableSceneLights
        state["cullBackfaces"] = self._cullBackfaces
        state["showInactivePrims"] = self._showInactivePrims
        state["showAllPrototypePrims"] = self._showAllPrototypePrims
        state["showAllMasterPrims"] = self._showAllPrototypePrims
        state["showUndefinedPrims"] = self._showUndefinedPrims
        state["showAbstractPrims"] = self._showAbstractPrims
        state["showPrimDisplayNames"] = self._showPrimDisplayNames
        state["rolloverPrimInfo"] = self._rolloverPrimInfo
        state["cameraOracles"] = self._displayCameraOracles
        state["cameraMaskMode"] = self._cameraMaskMode
        state["cameraMaskOutline"] = self._showMask_Outline
        state["cameraReticlesInside"] = self._showReticles_Inside
        state["cameraReticlesOutside"] = self._showReticles_Outside
        state["showHUD"] = self._showHUD
        state["showHUDInfo"] = self._showHUD_Info
        state["showHUDComplexity"] = self._showHUD_Complexity
        state["showHUDPerformance"] = self._showHUD_Performance
        state["showHUDGPUStats"] = self._showHUD_GPUstats
        state["fontSize"] = self._fontSize
        state["appIconMode"] = self._appIconMode

    @property
    def cameraMaskColor(self):
        return self._cameraMaskColor

    @cameraMaskColor.setter
    @visibleViewSetting
    def cameraMaskColor(self, color):
        self._cameraMaskColor = color

    @property
    def cameraReticlesColor(self):
        return self._cameraReticlesColor

    @cameraReticlesColor.setter
    @visibleViewSetting
    def cameraReticlesColor(self, color):
        self._cameraReticlesColor = color

    @property
    def defaultMaterialAmbient(self):
        return self._defaultMaterialAmbient

    @defaultMaterialAmbient.setter
    @visibleViewSetting
    def defaultMaterialAmbient(self, value):
        if value != self._defaultMaterialAmbient:
            self._defaultMaterialAmbient = value
            self.signalDefaultMaterialChanged.emit()

    @property
    def defaultMaterialSpecular(self):
        return self._defaultMaterialSpecular

    @defaultMaterialSpecular.setter
    @visibleViewSetting
    def defaultMaterialSpecular(self, value):
        if value != self._defaultMaterialSpecular:
            self._defaultMaterialSpecular = value
            self.signalDefaultMaterialChanged.emit()

    @visibleViewSetting
    def setDefaultMaterial(self, ambient, specular):
        if (ambient != self._defaultMaterialAmbient
                or specular != self._defaultMaterialSpecular):
            self._defaultMaterialAmbient = ambient
            self._defaultMaterialSpecular = specular
            self.signalDefaultMaterialChanged.emit()

    def resetDefaultMaterial(self):
        self.setDefaultMaterial(DEFAULT_AMBIENT, DEFAULT_SPECULAR)

    @property
    def complexity(self):
        return self._complexity

    @complexity.setter
    @visibleViewSetting
    def complexity(self, value):
        if value not in RefinementComplexities.ordered():
            raise ValueError("Expected Complexity, got: '{}'.".format(value))
        self._complexity = value

    @property
    def renderMode(self):
        return self._renderMode

    @renderMode.setter
    @visibleViewSetting
    def renderMode(self, value):
        self._renderMode = value

    @property
    def freeCameraFOV(self):
        return self._freeCameraFOV

    @freeCameraFOV.setter
    @freeCameraViewSetting
    def freeCameraFOV(self, value):
        if self._freeCamera:
            # Setting the freeCamera's fov will trigger our own update
            self._freeCamera.fov = value
        else:
            self._freeCameraFOV = value

    @property
    def freeCameraOverrideNear(self):
        """Returns the free camera's near clipping plane value, if it has been
        overridden by the user. Returns None if there is no user-defined near
        clipping plane."""
        return self._freeCameraOverrideNear

    @freeCameraOverrideNear.setter
    @freeCameraViewSetting
    def freeCameraOverrideNear(self, value):
        """Sets the near clipping plane to the given value. Passing in None will
        clear the current override."""
        self._freeCameraOverrideNear = value
        if self._freeCamera:
            self._freeCamera.overrideNear = value

    @property
    def freeCameraOverrideFar(self):
        """Returns the free camera's far clipping plane value, if it has been
        overridden by the user. Returns None if there is no user-defined far
        clipping plane."""
        return self._freeCameraOverrideFar

    @freeCameraOverrideFar.setter
    @freeCameraViewSetting
    def freeCameraOverrideFar(self, value):
        """Sets the far clipping plane to the given value. Passing in None will
        clear the current override."""
        self._freeCameraOverrideFar = value
        if self._freeCamera:
            self._freeCamera.overrideFar = value

    @property
    def freeCameraAspect(self):
        return self._freeCameraAspect
    
    @freeCameraAspect.setter
    @freeCameraViewSetting
    def freeCameraAspect(self, value):
        if self._freeCamera:
            # Setting the freeCamera's aspect ratio will trigger our own update
            self._freeCamera.aspectRatio = value
        else:
            self._freeCameraAspect = value

    def _frustumChanged(self):
        """
        Needed when updating any camera setting (including movements). Will not
        update the property viewer.
        """
        self.signalFreeCameraSettingChanged.emit()

    def _frustumSettingsChanged(self):
        """
        Needed when updating specific camera settings (e.g., aperture). See
        _updateFreeCameraData for the full list of dependent settings. Will
        update the property viewer.
        """
        self._updateFreeCameraData()
        self.signalSettingChanged.emit()

    def _updateFreeCameraData(self):
        '''Updates member variables with the current free camera view settings.
        '''
        if self._freeCamera:
            self._freeCameraFOV = self.freeCamera.fov
            self._freeCameraOverrideNear = self.freeCamera.overrideNear
            self._freeCameraOverrideFar = self.freeCamera.overrideFar
            if self._lockFreeCameraAspect:
                self._freeCameraAspect = self.freeCamera.aspectRatio

    @property
    def lockFreeCameraAspect(self):
        return self._lockFreeCameraAspect
    
    @lockFreeCameraAspect.setter
    @visibleViewSetting
    def lockFreeCameraAspect(self, value):
        self._lockFreeCameraAspect = value

        if value and not self.showMask:
            # Make sure the camera mask is turned on so the locked aspect ratio
            # is visible in the viewport.
            self.cameraMaskMode = CameraMaskModes.FULL

    @property
    def colorCorrectionMode(self):
        return self._colorCorrectionMode

    @colorCorrectionMode.setter
    @visibleViewSetting
    def colorCorrectionMode(self, value):
        self._colorCorrectionMode = value

    @property
    def ocioSettings(self):
        return self._ocioSettings

    @visibleViewSetting
    def setOcioSettings(self, colorSpace="", display="", view=""):
        """Specifies the OCIO settings to be used. Setting the OCIO 'display'
           requires a 'view' to be specified."""

        if colorSpace:
            self._ocioSettings._colorSpace = colorSpace

        if display:
            if view:
                self._ocioSettings._display = display
                self._ocioSettings._view = view
            else:
                PrintWarning("Cannot set a OCIO display without a view."\
                             "Using default settings instead.")
                self._ocioSettings._display = ""
                self._ocioSettings._view = ""

    @property
    def pickMode(self):
        return self._pickMode

    @pickMode.setter
    @invisibleViewSetting
    def pickMode(self, value):
        self._pickMode = value

    @property
    def showAABBox(self):
        return self._showAABBox

    @showAABBox.setter
    @visibleViewSetting
    def showAABBox(self, value):
        self._showAABBox = value

    @property
    def showOBBox(self):
        return self._showOBBox

    @showOBBox.setter
    @visibleViewSetting
    def showOBBox(self, value):
        self._showOBBox = value

    @property
    def showBBoxes(self):
        return self._showBBoxes

    @showBBoxes.setter
    @visibleViewSetting
    def showBBoxes(self, value):
        self._showBBoxes = value

    @property
    def autoComputeClippingPlanes(self):
        return self._autoComputeClippingPlanes

    @autoComputeClippingPlanes.setter
    @visibleViewSetting
    def autoComputeClippingPlanes(self, value):
        self._autoComputeClippingPlanes = value
        self.signalAutoComputeClippingPlanesChanged.emit()

    @property
    def showBBoxPlayback(self):
        return self._showBBoxPlayback

    @showBBoxPlayback.setter
    @visibleViewSetting
    def showBBoxPlayback(self, value):
        self._showBBoxPlayback = value

    @property
    def displayGuide(self):
        return self._displayGuide

    @displayGuide.setter
    @visibleViewSetting
    def displayGuide(self, value):
        self._displayGuide = value

    @property
    def displayProxy(self):
        return self._displayProxy

    @displayProxy.setter
    @visibleViewSetting
    def displayProxy(self, value):
        self._displayProxy = value

    @property
    def displayRender(self):
        return self._displayRender

    @displayRender.setter
    @visibleViewSetting
    def displayRender(self, value):
        self._displayRender = value

    @property
    def displayCameraOracles(self):
        return self._displayCameraOracles

    @displayCameraOracles.setter
    @visibleViewSetting
    def displayCameraOracles(self, value):
        self._displayCameraOracles = value

    @property
    def enableSceneMaterials(self):
        return self._enableSceneMaterials

    @enableSceneMaterials.setter
    @visibleViewSetting
    def enableSceneMaterials(self, value):
        self._enableSceneMaterials = value

    @property
    def enableSceneLights(self):
        return self._enableSceneLights

    @enableSceneLights.setter
    @visibleViewSetting
    def enableSceneLights(self, value):
        self._enableSceneLights = value

    @property
    def cullBackfaces(self):
        return self._cullBackfaces

    @cullBackfaces.setter
    @visibleViewSetting
    def cullBackfaces(self, value):
        self._cullBackfaces = value

    @property
    def showInactivePrims(self):
        return self._showInactivePrims

    @showInactivePrims.setter
    @invisibleViewSetting
    def showInactivePrims(self, value):
        self._showInactivePrims = value

    @property
    def showAllPrototypePrims(self):
        return self._showAllPrototypePrims

    @showAllPrototypePrims.setter
    @invisibleViewSetting
    def showAllPrototypePrims(self, value):
        self._showAllPrototypePrims = value

    @property
    def showUndefinedPrims(self):
        return self._showUndefinedPrims

    @showUndefinedPrims.setter
    @invisibleViewSetting
    def showUndefinedPrims(self, value):
        self._showUndefinedPrims = value

    @property
    def showAbstractPrims(self):
        return self._showAbstractPrims

    @showAbstractPrims.setter
    @invisibleViewSetting
    def showAbstractPrims(self, value):
        self._showAbstractPrims = value

    @property
    def showPrimDisplayNames(self):
        return self._showPrimDisplayNames

    @showPrimDisplayNames.setter
    @invisibleViewSetting
    def showPrimDisplayNames(self, value):
        self._showPrimDisplayNames = value

    @property
    def rolloverPrimInfo(self):
        return self._rolloverPrimInfo

    @rolloverPrimInfo.setter
    @invisibleViewSetting
    def rolloverPrimInfo(self, value):
        self._rolloverPrimInfo = value

    @property
    def cameraMaskMode(self):
        return self._cameraMaskMode

    @cameraMaskMode.setter
    @visibleViewSetting
    def cameraMaskMode(self, value):
        self._cameraMaskMode = value

    @property
    def showMask(self):
        return self._cameraMaskMode in (CameraMaskModes.FULL, CameraMaskModes.PARTIAL)

    @property
    def showMask_Opaque(self):
        return self._cameraMaskMode == CameraMaskModes.FULL

    @property
    def showMask_Outline(self):
        return self._showMask_Outline

    @showMask_Outline.setter
    @visibleViewSetting
    def showMask_Outline(self, value):
        self._showMask_Outline = value

    @property
    def showReticles_Inside(self):
        return self._showReticles_Inside

    @showReticles_Inside.setter
    @visibleViewSetting
    def showReticles_Inside(self, value):
        self._showReticles_Inside = value

    @property
    def showReticles_Outside(self):
        return self._showReticles_Outside

    @showReticles_Outside.setter
    @visibleViewSetting
    def showReticles_Outside(self, value):
        self._showReticles_Outside = value

    @property
    def showHUD(self):
        return self._showHUD

    @showHUD.setter
    @visibleViewSetting
    def showHUD(self, value):
        self._showHUD = value

    @property
    def showHUD_Info(self):
        return self._showHUD_Info

    @showHUD_Info.setter
    @visibleViewSetting
    def showHUD_Info(self, value):
        self._showHUD_Info = value

    @property
    def showHUD_Complexity(self):
        return self._showHUD_Complexity

    @showHUD_Complexity.setter
    @visibleViewSetting
    def showHUD_Complexity(self, value):
        self._showHUD_Complexity = value

    @property
    def showHUD_Performance(self):
        return self._showHUD_Performance

    @showHUD_Performance.setter
    @visibleViewSetting
    def showHUD_Performance(self, value):
        self._showHUD_Performance = value

    @property
    def showHUD_GPUstats(self):
        return self._showHUD_GPUstats

    @showHUD_GPUstats.setter
    @visibleViewSetting
    def showHUD_GPUstats(self, value):
        self._showHUD_GPUstats = value

    @property
    def ambientLightOnly(self):
        return self._ambientLightOnly

    @ambientLightOnly.setter
    @visibleViewSetting
    def ambientLightOnly(self, value):
        self._ambientLightOnly = value

    @property
    def domeLightEnabled(self):
        return self._domeLightEnabled

    @domeLightEnabled.setter
    @visibleViewSetting
    def domeLightEnabled(self, value):
        self._domeLightEnabled = value

    @property
    def domeLightTexturesVisible(self):
        return self._domeLightTexturesVisible

    @domeLightTexturesVisible.setter
    @visibleViewSetting
    def domeLightTexturesVisible(self, value):
        self._domeLightTexturesVisible = value

    @property
    def clearColorText(self):
        return self._clearColorText

    @clearColorText.setter
    @visibleViewSetting
    def clearColorText(self, value):
        if value not in ClearColors:
            raise ValueError("Unknown clear color: '{}'".format(value))
        self._clearColorText = value

    @property
    def clearColor(self):
        return _CLEAR_COLORS_DICT[self._clearColorText]

    @property
    def highlightColorName(self):
        return self._highlightColorName

    @highlightColorName.setter
    @visibleViewSetting
    def highlightColorName(self, value):
        if value not in HighlightColors:
            raise ValueError("Unknown highlight color: '{}'".format(value))
        self._highlightColorName = value

    @property
    def highlightColor(self):
        return _HIGHLIGHT_COLORS_DICT[self._highlightColorName]

    @property
    def selHighlightMode(self):
        return self._selHighlightMode

    @selHighlightMode.setter
    @visibleViewSetting
    def selHighlightMode(self, value):
        if value not in SelectionHighlightModes:
            raise ValueError("Unknown highlight mode: '{}'".format(value))
        self._selHighlightMode = value

    @property
    def redrawOnScrub(self):
        return self._redrawOnScrub

    @redrawOnScrub.setter
    @visibleViewSetting
    def redrawOnScrub(self, value):
        self._redrawOnScrub = value

    @property
    def stepSize(self):
        return self._stepSize

    @stepSize.setter
    @visibleViewSetting
    def stepSize(self, value):
        self._stepSize = value

    @property
    def freeCamera(self):
        return self._freeCamera

    @freeCamera.setter
    @visibleViewSetting
    def freeCamera(self, value):
        # ViewSettingsDataModel does not guarantee it will hold a valid
        # FreeCamera, but if one is set, we will keep the dataModel's stateful
        # free camera settings (fov, clipping planes, aspect ratio) in sync with
        # the FreeCamera

        if not isinstance(value, FreeCamera) and value != None:
            raise TypeError("Free camera must be a FreeCamera object.")
        if self._freeCamera:
            self._freeCamera.signalFrustumChanged.disconnect(
                self._frustumChanged)
            self._freeCamera.signalFrustumSettingsChanged.disconnect(
                self._frustumSettingsChanged)
        self._freeCamera = value
        if self._freeCamera:
            self._freeCamera.signalFrustumChanged.connect(
                self._frustumChanged)
            self._freeCamera.signalFrustumSettingsChanged.connect(
                self._frustumSettingsChanged)
            self._updateFreeCameraData()

    @property
    def cameraPath(self):
        return self._cameraPath

    @cameraPath.setter
    @visibleViewSetting
    def cameraPath(self, value):
        if ((not isinstance(value, Sdf.Path) or not value.IsPrimPath())
                and value is not None):
            raise TypeError("Expected prim path, got: {}".format(value))
        self._cameraPath = value

    @property
    def cameraPrim(self):
        if self.cameraPath is not None and self._rootDataModel.stage is not None:
            return self._rootDataModel.stage.GetPrimAtPath(self.cameraPath)
        else:
            return None

    @cameraPrim.setter
    def cameraPrim(self, value):
        if value is not None:
            if value.IsA(UsdGeom.Camera):
                self.cameraPath = value.GetPrimPath()
            else:
                PrintWarning("Incorrect Prim Type",
                    "Attempted to view the scene using the prim '%s', but "
                    "the prim is not a UsdGeom.Camera." % (value.GetName()))
        else:
            self.cameraPath = None

    @property
    def fontSize(self):
        return self._fontSize

    @fontSize.setter
    @visibleViewSetting
    def fontSize(self, value):
        if value != self._fontSize:
            self._fontSize = value
            self.signalStyleSettingsChanged.emit()

    @property
    def appIconMode(self):
        return self._appIconMode

    @appIconMode.setter
    @invisibleViewSetting
    def appIconMode(self, value):
        if value != self._appIconMode:
            self._appIconMode = value
            self.signalStyleSettingsChanged.emit()




#
# Copyright 2019 Pixar
#
# Licensed under the terms set forth in the LICENSE.txt file available at
# https://openusd.org/license.
#

class RefinementComplexities(object):
    """
    An enum-like container of standard complexity settings.
    """

    class _RefinementComplexity(object):
        """
        Class which represents a level of mesh refinement complexity. Each
        level has a string identifier, a display name, and a float complexity
        value.
        """

        def __init__(self, compId, name, value):
            self._id = compId
            self._name = name
            self._value = value

        def __repr__(self):
            return self.id

        @property
        def id(self):
            return self._id

        @property
        def name(self):
            return self._name

        @property
        def value(self):
            return self._value

    LOW = _RefinementComplexity("low", "Low", 1.0)
    MEDIUM = _RefinementComplexity("medium", "Medium", 1.1)
    HIGH = _RefinementComplexity("high", "High", 1.2)
    VERY_HIGH = _RefinementComplexity("veryhigh", "Very High", 1.3)

    _ordered = (LOW, MEDIUM, HIGH, VERY_HIGH)

    @classmethod
    def ordered(cls):
        """
        Get a tuple of all complexity levels in order.
        """
        return cls._ordered

    @classmethod
    def fromId(cls, compId):
        """
        Get a complexity from its identifier.
        """
        matches = [comp for comp in cls._ordered if comp.id == compId]
        if len(matches) == 0:
            raise ValueError("No complexity with id '{}'".format(compId))
        return matches[0]

    @classmethod
    def fromName(cls, name):
        """
        Get a complexity from its display name.
        """
        matches = [comp for comp in cls._ordered if comp.name == name]
        if len(matches) == 0:
            raise ValueError("No complexity with name '{}'".format(name))
        return matches[0]

    @classmethod
    def next(cls, comp):
        """
        Get the next highest level of complexity. If already at the highest
        level, return it.
        """
        if comp not in cls._ordered:
            raise ValueError("Invalid complexity: {}".format(comp))
        nextIndex = min(
            len(cls._ordered) - 1,
            cls._ordered.index(comp) + 1)
        return cls._ordered[nextIndex]

    @classmethod
    def prev(cls, comp):
        """
        Get the next lowest level of complexity. If already at the lowest
        level, return it.
        """
        if comp not in cls._ordered:
            raise ValueError("Invalid complexity: {}".format(comp))
        prevIndex = max(0, cls._ordered.index(comp) - 1)
        return cls._ordered[prevIndex]


def AddCmdlineArgs(argsParser, defaultValue=RefinementComplexities.LOW,
        altHelpText=''):
    """
    Adds complexity-related command line arguments to argsParser.

    The resulting 'complexity' argument will be one of the standard
    RefinementComplexities.
    """
    helpText = altHelpText
    if not helpText:
        helpText = ('level of refinement to use (default=%(default)s)')

    argsParser.add_argument('--complexity', '-c', action='store',
        type=RefinementComplexities.fromId,
        default=defaultValue,
        choices=[c for c in RefinementComplexities.ordered()],
        help=helpText)



#
# Copyright 2022 Pixar
#
# Licensed under the terms set forth in the LICENSE.txt file available at
# https://openusd.org/license.
#

class FixBrokenPixarSchemas(object):
    """
    A class which takes a usdLayer and clients can apply appropriate fixes
    defined as utility methods of this class, example FixupMaterialBindingAPI.

    Every Fixup method iterates on each prim in the layer and applies specific
    fixes.
    """
    def __init__(self, usdLayer):
        self._usdLayer = usdLayer
        self._skelBindingAPIProps = None
        # Keeps track if the Layer was updated by any of the Fixer method
        self._layerUpdated = False

    def _ApplyAPI(self, listOp, apiSchema):
        if listOp.isExplicit:
            items = listOp.explicitItems
            items.append(apiSchema)
            listOp.explicitItems = items
        else:
            items = listOp.prependedItems
            items.append(apiSchema)
            listOp.prependedItems = items
        return listOp

    def IsLayerUpdated(self):
        """
        Returns the update status of the usdLayer, an instance of 
        FixBrokenPixarSchemas is holding. Fixer methods will set 
        self._layerUpdated to True if any of the Fixer methods applies fixes to 
        the layer.
        """
        return self._layerUpdated

    def FixupCoordSysAPI(self):
        """
        Makes sure CoordSysAPI multiapply schema is applied and the instanced
        binding relationship is used, instead of old non-applied CoordSysAPI
        "coordSys:name" binding.
        """
        def _PrimSpecProvidesCoordSysAPI(path):
            from pxr import Sdf
            if not path.IsPrimPath():
                return
            primSpec = self._usdLayer.GetPrimAtPath(path)
            apiSchemas = primSpec.GetInfo("apiSchemas")
            # Collect all relationship which use of UsdShadeCoordSysAPI encoding
            relationshipToUpdate = []
            missingApiSchema = []
            for rel in primSpec.relationships:
                relName = rel.name
                # is the relationship old style coordSys binding
                if (relName.startswith("coordSys:") and \
                        (not relName.endswith(":binding") or \
                            relName.count(":") == 1)):
                    relationshipToUpdate.append(rel)
                # Since there was no prior codepath that could have left us in a
                # state where a "coordSys:<name>:binding" was added but no
                # UsdShadeCoordSysAPI is applied with the "<name>", following
                # finds and fixes missing application of UsdShadeCoordSysAPI 
                # but when new encoding relationship is present on the spec.
                if (relName.startswith("coordSys:") and \
                    relName.endswith(":binding") and \
                    relName.count(":") > 1):
                    instanceName = rel.name.split("coordSys:")[-1]. \
                            split(":binding")[0]
                    coordSysAPIName = "CoordSysAPI:%s" %instanceName
                    if not apiSchemas.HasItem(coordSysAPIName):
                        missingApiSchema.append(coordSysAPIName)
                    

            if len(relationshipToUpdate) == 0 and len(missingApiSchema) == 0:
                return

            # Will definitely be doing some updating!
            self._layerUpdated = True

            # though it shouldn't matter for our fixup usage, but better to
            # follow best practice and make ALL changes in a change block.
            with Sdf.ChangeBlock():
                for rel in relationshipToUpdate:
                    instanceName = rel.name.split("coordSys:")[-1]
                    # Apply API
                    coordSysAPIName = "CoordSysAPI:%s" %instanceName
                    apiSchemas = self._ApplyAPI(apiSchemas, coordSysAPIName)
                    # New Rel
                    newRelPath = rel.path.ReplaceName(rel.name+":binding")
                    # CopySpec
                    Sdf.CopySpec(self._usdLayer, rel.path, 
                            self._usdLayer, newRelPath)
                    # Remove old rel
                    primSpec.RemoveProperty(rel)
                # Apply any missing UsdShadeCoordSysAPI api schema
                for missingAPI in missingApiSchema:
                    apiSchemas = self._ApplyAPI(apiSchemas, missingAPI)

                primSpec.SetInfo("apiSchemas", apiSchemas)

        self._usdLayer.Traverse("/", _PrimSpecProvidesCoordSysAPI)


    def FixupMaterialBindingAPI(self):
        """
        Makes sure MaterialBindingAPI is applied on the prim, which defines a
        material:binding property spec. Marks the layer updated if fixes are
        applied.
        """
        def _PrimSpecProvidesMaterialBinding(path):
            if not path.IsPrimPath():
                return
            primSpec = self._usdLayer.GetPrimAtPath(path)
            hasMaterialBindingRel = \
                    any(prop.name.startswith("material:binding") \
                        for prop in primSpec.properties)
            apiSchemas = primSpec.GetInfo("apiSchemas")
            hasMaterialBindingAPI = "MaterialBindingAPI" in \
                    apiSchemas.GetAddedOrExplicitItems()
            if hasMaterialBindingRel and not hasMaterialBindingAPI:
                self._layerUpdated = True
                newApiSchemas = self._ApplyAPI(apiSchemas, "MaterialBindingAPI")
                primSpec.SetInfo("apiSchemas", newApiSchemas)

        self._usdLayer.Traverse("/", _PrimSpecProvidesMaterialBinding)
        

    def FixupSkelBindingAPI(self):
        """
        Makes sure SkelBindingAPI is applied on the prim, which defines
        appropriate UsdSkel properties which are imparted by SkelBindingAPI.
        Marks the layer as updated if fixes are applied.
        """

        def _PrimSpecProvidesSkelBindingProperties(path):
            if not path.IsPrimPath():
                return

            # get skelBindingAPI props if not already populated
            if not self._skelBindingAPIProps:
                from pxr import Usd, UsdSkel
                usdSchemaRegistry = Usd.SchemaRegistry()
                primDef = usdSchemaRegistry.BuildComposedPrimDefinition("",
                        ["SkelBindingAPI"])
                self._skelBindingAPIProps = primDef.GetPropertyNames()
        
            primSpec = self._usdLayer.GetPrimAtPath(path)
            primSpecProps = [prop.name for prop in primSpec.properties]
            apiSchemas = primSpec.GetInfo("apiSchemas")
            hasSkelBindingAPI = "SkelBindingAPI" in \
                    apiSchemas.GetAddedOrExplicitItems()
            for skelProperty in self._skelBindingAPIProps:
                if (skelProperty in primSpecProps) and not hasSkelBindingAPI:
                    self._layerUpdated = True
                    newApiSchemas = self._ApplyAPI(apiSchemas, "SkelBindingAPI")
                    primSpec.SetInfo("apiSchemas", newApiSchemas)
                    return

        self._usdLayer.Traverse("/", _PrimSpecProvidesSkelBindingProperties)


    def FixupUpAxis(self):
        """
        Makes sure the layer specifies a upAxis metadata, and if not upAxis
        metadata is set to the default provided by UsdGeom. Marks the layer as 
        updated if fixes are applied.
        """
        from pxr import Usd, UsdGeom
        usdStage = Usd.Stage.Open(self._usdLayer)
        if not usdStage.HasAuthoredMetadata(UsdGeom.Tokens.upAxis):
            self._layerUpdated = True
            usdStage.SetMetadata(UsdGeom.Tokens.upAxis, 
                UsdGeom.GetFallbackUpAxis())


    def FixupRenderSettingsTerminalsAPI(self):
        """
        Makes sure that RenderSettings terminals targeting display filters,
        integrators, and sample filters are relationships instead of output
        connections. Marks the layer as updated if fixes are applied.
        """
        replaceAttrs = ["outputs:ri:displayFilters",
                        "outputs:ri:sampleFilters",
                        "outputs:ri:integrator"]

        def _RenderSettingsTerminalAPIUsesRelationships(path):
            if not path.IsPrimPath():
                return
        
            from pxr import Usd, Sdf
            primSpec = self._usdLayer.GetPrimAtPath(path)
            if primSpec.typeName != "RenderSettings":
                return

            registry = Usd.SchemaRegistry()
            primDef = registry.FindConcretePrimDefinition(primSpec.typeName)
            if "PxrRenderTerminalsAPI" not in primDef.GetAppliedAPISchemas():
                return

            for attr in primSpec.attributes:
                attrName = attr.name
                if attrName in replaceAttrs:
                    conns = attr.connectionPathList.GetAppliedItems()
                    if conns:
                        self._layerUpdated = True
                        relName = attrName.replace("outputs:", "")
                        relPath = primSpec.path.AppendProperty(relName)
                        rel = Sdf.CreateRelationshipInLayer(self._usdLayer,
                            relPath, Sdf.VariabilityUniform, attr.custom)
                        rel.targetPathList.ClearEditsAndMakeExplicit()
                        primPaths = [p.GetPrimPath() for p in conns]
                        rel.targetPathList.explicitItems = primPaths
                        primSpec.RemoveProperty(attr)

        self._usdLayer.Traverse("/", _RenderSettingsTerminalAPIUsesRelationships)


    def FixupCameraProjectionAPI(self):
        """
        Makes sure that PxrCameraProjectionAPI supplies a relationship for
        the ri:projection to be used with a projection plugin target, instead of
        an output connection. Marks the layer as updated if fixes are applied.
        """
        replaceAttr = "outputs:ri:projection"

        def _CameraProjectionAPIUsesRelationship(path):
            if not path.IsPrimPath():
                return

            from pxr import Usd, Sdf
            primSpec = self._usdLayer.GetPrimAtPath(path)
            # Since PxrCameraProjectionAPI is only applied to Camera prims,
            # early out if not a Camera prim.
            if primSpec.typeName != "Camera":
                return

            registry = Usd.SchemaRegistry()
            primDef = registry.FindConcretePrimDefinition(primSpec.typeName)
            if "PxrCameraProjectionAPI" not in primDef.GetAppliedAPISchemas():
                return

            if replaceAttr not in [attr.name for attr in primSpec.attributes]:
                return

            attr = primSpec.GetAttribute(replaceAttr)
            conns = attr.connectionPathList.GetAppliedItems()
            if not conns:
                return

            self._layerUpdated = True
            relName = replaceAttr.replace("outputs:", "")
            relPath = primSpec.path.AppendProperty(relName)
            rel = Sdf.CreateRelationshipInLayer(self._usdLayer,
                relPath, Sdf.VariabilityUniform, attr.custom)
            rel.targetPathList.ClearEditsAndMakeExplicit()
            primPaths = [p.GetPrimPath() for p in conns]
            rel.targetPathList.explicitItems = primPaths
            primSpec.RemoveProperty(attr)




#
# Copyright 2016 Pixar
#
# Licensed under the terms set forth in the LICENSE.txt file available at
# https://openusd.org/license.
#

from __future__ import division
from .qt import QtCore, QtWidgets

# simple class to have a "clear" button on a line edit when the line edit
# contains an override. Clicking the clear button returns to the default value.

class OverridableLineEdit(QtWidgets.QLineEdit):
    def __init__(self, parent):
        QtWidgets.QLineEdit.__init__(self, parent)

        # create the clear button.
        self._clearButton = QtWidgets.QToolButton(self)
        self._clearButton.setText('x')
        self._clearButton.setCursor(QtCore.Qt.CursorShape.ArrowCursor)
        self._clearButton.setFixedSize(16, 16)
        self._clearButton.hide()
        self._defaultText = ''  # default value holder

        self._clearButton.clicked.connect(self._resetDefault)
        self.textEdited.connect(self._overrideSet)

    # properly place the button
    def resizeEvent(self, event):
        sz = QtCore.QSize(self._clearButton.size())
        frameWidth = self.style().pixelMetric(QtWidgets.QStyle.PixelMetric.PM_DefaultFrameWidth)
        self._clearButton.move(self.rect().right() - frameWidth - sz.width(),
                               (self.rect().bottom() + 1 - sz.height())/2)

    # called when the user types an override
    def _overrideSet(self, text):
        self._clearButton.setVisible(True)

    # called programatically to reset the default text
    def setText(self, text):
        QtWidgets.QLineEdit.setText(self, text)
        self._defaultText = text

        self._clearButton.setVisible(False)

    # called when the clear button is clicked
    def _resetDefault(self):
        self.setText(self._defaultText)





#
# Copyright 2016 Pixar
#
# Licensed under the terms set forth in the LICENSE.txt file available at
# https://openusd.org/license.
#
#
# A base class for all context menu items.
# This provides a simple behavior to ensure that a chosen
# context menu item is valid. This helps us avoid a situation
# in which a user right-clicks in an area with no item but still
# receives a context menu.
#
class UsdviewContextMenuItem():
    def isValid(self):
        ''' Menu items which have an invalid internal item are considered invalid.
            Header menus don't contain an internal _item attribute, so we
            return true in the case of the attribute being undefined.
            We use this function to give this state a clearer name.
        '''
        return not hasattr(self, "_item") or self._item is not None



#
# Copyright 2016 Pixar
#
# Licensed under the terms set forth in the LICENSE.txt file available at
# https://openusd.org/license.
#
from .qt import QtCore, QtWidgets
from .usdviewContextMenuItem import UsdviewContextMenuItem

#
# Specialized context menu for adding and removing columns
# in the prim browser and attribute inspector.
#
class HeaderContextMenu(QtWidgets.QMenu):

    def __init__(self, parent):
        QtWidgets.QMenu.__init__(self, "Columns", parent)
        self._menuItems = _GetContextMenuItems(parent)

        for menuItem in self._menuItems:
            if menuItem.isValid():
                # create menu actions.  Associate them with each Item, so that
                # we can update them when we show, since column visibility can
                # change.
                action = self.addAction(menuItem.GetText(), menuItem.RunCommand)
                action.setCheckable(True)
                menuItem.action = action

        self.aboutToShow.connect(self._prepForShow)

    def _prepForShow(self):
        for menuItem in self._menuItems:
            if menuItem.action:
                menuItem.action.setChecked(menuItem.IsChecked())
                menuItem.action.setEnabled(menuItem.IsEnabled())


def _GetContextMenuItems(parent):
    # create a list of HeaderContextMenuItem classes
    # initialized with the parent object and column
    itemList = []

    return [HeaderContextMenuItem(parent, column) \
                for column in range(parent.columnCount())]

# The base class for header context menus
class HeaderContextMenuItem(UsdviewContextMenuItem):

    def __init__(self, parent, column):
        self._parent = parent
        self._column = column
        self._action = None

        if isinstance(parent, QtWidgets.QTreeWidget):
            self._text = parent.headerItem().text(column)
        else:
            self._text = parent.horizontalHeaderItem(column).text()

    def GetText(self):
        # returns the text to be displayed in menu
        return self._text

    def IsEnabled(self):
        # Enable context menu item for columns except the "Name" column.
        return 'Name' not in self.GetText()

    def IsChecked(self):
        # true if the column is visible, false otherwise
        return not self._parent.isColumnHidden(self._column)

    def RunCommand(self):
        # show or hide the column depending on its previous state
        self._parent.setColumnHidden(self._column, self.IsChecked())

    @property
    def action(self):
        return self._action

    @action.setter
    def action(self, action):
        self._action = action




# -*- coding: utf-8 -*-

################################################################################
## Form generated from reading UI file 'preferencesUI.ui'
##
## Created by: Qt User Interface Compiler version 6.11.2
##
## WARNING! All changes made in this file will be lost when recompiling UI file!
################################################################################

from PySide6.QtCore import (QCoreApplication, QDate, QDateTime, QLocale,
    QMetaObject, QObject, QPoint, QRect,
    QSize, QTime, QUrl, Qt)
from PySide6.QtGui import (QBrush, QColor, QConicalGradient, QCursor,
    QFont, QFontDatabase, QGradient, QIcon,
    QImage, QKeySequence, QLinearGradient, QPainter,
    QPalette, QPixmap, QRadialGradient, QTransform)
from PySide6.QtWidgets import (QAbstractButton, QApplication, QComboBox, QDialog,
    QDialogButtonBox, QFrame, QHBoxLayout, QLabel,
    QSizePolicy, QSpacerItem, QSpinBox, QVBoxLayout,
    QWidget)

class Ui_Preferences(object):
    def setupUi(self, Preferences):
        if not Preferences.objectName():
            Preferences.setObjectName(u"Preferences")
        Preferences.resize(295, 135)
        self.verticalLayout = QVBoxLayout(Preferences)
        self.verticalLayout.setObjectName(u"verticalLayout")
        self.prefsOverButtonsLayout = QVBoxLayout()
        self.prefsOverButtonsLayout.setObjectName(u"prefsOverButtonsLayout")
        self.horizontalLayout_3 = QHBoxLayout()
        self.horizontalLayout_3.setObjectName(u"horizontalLayout_3")
        self.fontSizeLabel = QLabel(Preferences)
        self.fontSizeLabel.setObjectName(u"fontSizeLabel")

        self.horizontalLayout_3.addWidget(self.fontSizeLabel)

        self.fontSizeSpinBox = QSpinBox(Preferences)
        self.fontSizeSpinBox.setObjectName(u"fontSizeSpinBox")
        self.fontSizeSpinBox.setMinimum(6)
        self.fontSizeSpinBox.setValue(10)

        self.horizontalLayout_3.addWidget(self.fontSizeSpinBox)

        self.horizontalSpacer_2 = QSpacerItem(40, 20, QSizePolicy.Policy.Expanding, QSizePolicy.Policy.Minimum)

        self.horizontalLayout_3.addItem(self.horizontalSpacer_2)


        self.prefsOverButtonsLayout.addLayout(self.horizontalLayout_3)

        self.horizontalLayout_4 = QHBoxLayout()
        self.horizontalLayout_4.setObjectName(u"horizontalLayout_4")
        self.appIconLabel = QLabel(Preferences)
        self.appIconLabel.setObjectName(u"appIconLabel")

        self.horizontalLayout_4.addWidget(self.appIconLabel)

        self.appIconComboBox = QComboBox(Preferences)
        self.appIconComboBox.addItem("")
        self.appIconComboBox.addItem("")
        self.appIconComboBox.addItem("")
        self.appIconComboBox.addItem("")
        self.appIconComboBox.setObjectName(u"appIconComboBox")

        self.horizontalLayout_4.addWidget(self.appIconComboBox)

        self.horizontalSpacer_3 = QSpacerItem(40, 20, QSizePolicy.Policy.Expanding, QSizePolicy.Policy.Minimum)

        self.horizontalLayout_4.addItem(self.horizontalSpacer_3)


        self.prefsOverButtonsLayout.addLayout(self.horizontalLayout_4)

        self.verticalSpacer = QSpacerItem(20, 40, QSizePolicy.Policy.Minimum, QSizePolicy.Policy.Expanding)

        self.prefsOverButtonsLayout.addItem(self.verticalSpacer)

        self.line = QFrame(Preferences)
        self.line.setObjectName(u"line")
        self.line.setFrameShape(QFrame.Shape.HLine)
        self.line.setFrameShadow(QFrame.Shadow.Sunken)

        self.prefsOverButtonsLayout.addWidget(self.line)

        self.horizontalLayout_2 = QHBoxLayout()
        self.horizontalLayout_2.setObjectName(u"horizontalLayout_2")
        self.horizontalSpacer = QSpacerItem(40, 20, QSizePolicy.Policy.Expanding, QSizePolicy.Policy.Minimum)

        self.horizontalLayout_2.addItem(self.horizontalSpacer)

        self.buttonBox = QDialogButtonBox(Preferences)
        self.buttonBox.setObjectName(u"buttonBox")
        self.buttonBox.setStandardButtons(QDialogButtonBox.Apply|QDialogButtonBox.Cancel|QDialogButtonBox.Ok)

        self.horizontalLayout_2.addWidget(self.buttonBox)


        self.prefsOverButtonsLayout.addLayout(self.horizontalLayout_2)


        self.verticalLayout.addLayout(self.prefsOverButtonsLayout)


        self.retranslateUi(Preferences)

        QMetaObject.connectSlotsByName(Preferences)
    # setupUi

    def retranslateUi(self, Preferences):
        Preferences.setWindowTitle(QCoreApplication.translate("Preferences", u"Preferences", None))
        Preferences.setProperty(u"comment", QCoreApplication.translate("Preferences", u"\n"
"     Copyright 2020 Pixar                                                                   \n"
"                                                                                            \n"
"     Licensed under the terms set forth in the LICENSE.txt file available at\n"
"     https://openusd.org/license.\n"
"  ", None))
        self.fontSizeLabel.setText(QCoreApplication.translate("Preferences", u"Font Size", None))
        self.appIconLabel.setText(QCoreApplication.translate("Preferences", u"App Icon", None))
        self.appIconComboBox.setItemText(0, QCoreApplication.translate("Preferences", u"Automatic", None))
        self.appIconComboBox.setItemText(1, QCoreApplication.translate("Preferences", u"Light", None))
        self.appIconComboBox.setItemText(2, QCoreApplication.translate("Preferences", u"Dark", None))
        self.appIconComboBox.setItemText(3, QCoreApplication.translate("Preferences", u"Blue", None))

    # retranslateUi




#
# Copyright 2024 Pixar
#
# Licensed under the terms set forth in the LICENSE.txt file available at
# https://openusd.org/license.
#
from pxr import Usd, UsdUtils, Usdviewq

def testUsdviewInputFunction(appController):
    with Usdviewq.Timer("traverse stage", True):
        stage = appController._dataModel.stage
        for prim in stage.Traverse():
            pass  # Just traverse all prims




#
# Copyright 2018 Pixar
#
# Licensed under the terms set forth in the LICENSE.txt file available at
# https://openusd.org/license.
#

# This file makes this module importable from python, which avoids a warning 
# when initializing the SdrRegistry.




#
# Copyright 2016 Pixar
#
# Licensed under the terms set forth in the LICENSE.txt file available at
# https://openusd.org/license.
#
from __future__ import print_function

from .qt import QtCore, QtGui, QtWidgets
from .usdviewContextMenuItem import UsdviewContextMenuItem
import os, subprocess, sys
from pxr import Ar
from pxr.UsdUtils.toolPaths import FindUsdBinary

#
# Specialized context menu for running commands in the layer stack view.
#
class LayerStackContextMenu(QtWidgets.QMenu):

    def __init__(self, parent, item):
        QtWidgets.QMenu.__init__(self, parent)
        self._menuItems = _GetContextMenuItems(item)

        for menuItem in self._menuItems:
            if menuItem.isValid():
                # create menu actions
                action = self.addAction(menuItem.GetText(), menuItem.RunCommand)

                # set enabled
                if not menuItem.IsEnabled():
                    action.setEnabled(False)


def _GetContextMenuItems(item):
    return [OpenLayerMenuItem(item),
            UsdviewLayerMenuItem(item),
            CopyLayerPathMenuItem(item),
            CopyLayerIdentifierMenuItem(item),
            CopyPathMenuItem(item),
            MuteOrUnmuteLayerMenuItem(item)]

#
# The base class for layer stack context menu items.
#
class LayerStackContextMenuItem(UsdviewContextMenuItem):

    def __init__(self, item):
        self._item = item

    def IsEnabled(self):
        return True

    def GetText(self):
        return ""

    def RunCommand(self):
        pass

#
# Opens the layer using usdedit.
#
class OpenLayerMenuItem(LayerStackContextMenuItem):
    def GetText(self):
        from .common import PrettyFormatSize
        fileSize = 0
        if (hasattr(self._item, "layerPath")
           and os.path.isfile(getattr(self._item, "layerPath"))):
            fileSize = os.path.getsize(getattr(self._item, "layerPath"))
        if fileSize:
            return "Open Layer In Editor (%s)" % PrettyFormatSize(fileSize)
        else:
            return "Open Layer In Editor"

    def IsEnabled(self):
        return hasattr(self._item, "layerPath")

    def RunCommand(self):
        if not self._item:
            return

        # Get layer path from item
        layerPath = getattr(self._item, "layerPath")
        if not layerPath:
            print("Error: Could not find layer file.")
            return

        if Ar.IsPackageRelativePath(layerPath):
            layerName = os.path.basename(
                Ar.SplitPackageRelativePathInner(layerPath)[1])
        else:
            layerName = os.path.basename(layerPath)
        
        layerName += ".tmp"

        usdeditExe = FindUsdBinary('usdedit')
        if not usdeditExe:
            print("Warning: Could not find 'usdedit', expected it to be in PATH.")
            return

        print("Opening file: %s" % layerPath)

        command =  [usdeditExe,'-n',layerPath,'-p',layerName]

        subprocess.Popen(command, close_fds=True)

#
# Opens the layer using usdview.
#
class UsdviewLayerMenuItem(LayerStackContextMenuItem):

    def GetText(self):
        return "Open Layer In usdview"

    def IsEnabled(self):
        return hasattr(self._item, "layerPath")

    def RunCommand(self):
        if not self._item:
            return

        # Get layer path from item
        layerPath = getattr(self._item, "layerPath")
        if not layerPath:
            return

        print("Spawning usdview %s" % layerPath)
        os.system("usdview %s &" % layerPath)

#
# Copy the layer path to clipboard
#
class CopyLayerPathMenuItem(LayerStackContextMenuItem):
    def GetText(self):
        return "Copy Layer Path"

    def RunCommand(self):
        if not self._item:
            return

        layerPath = getattr(self._item, "layerPath")
        if not layerPath:
            return

        cb = QtWidgets.QApplication.clipboard()
        cb.setText(layerPath, QtGui.QClipboard.Mode.Selection )
        cb.setText(layerPath, QtGui.QClipboard.Mode.Clipboard )

#
# Copy the layer identifier to clipboard
#
class CopyLayerIdentifierMenuItem(LayerStackContextMenuItem):
    def GetText(self):
        return "Copy Layer Identifier"

    def RunCommand(self):
        if not self._item:
            return

        identifier = getattr(self._item, "identifier")
        if not identifier:
            return

        cb = QtWidgets.QApplication.clipboard()
        cb.setText(identifier, QtGui.QClipboard.Mode.Selection )
        cb.setText(identifier, QtGui.QClipboard.Mode.Clipboard )

#
# Copy the prim path to clipboard, if there is one
#
class CopyPathMenuItem(LayerStackContextMenuItem):
    def GetText(self):
        return "Copy Object Path"

    def RunCommand(self):
        if not self._item:
            return

        path = getattr(self._item, "path")
        if not path:
            return

        path = str(path)
        cb = QtWidgets.QApplication.clipboard()
        cb.setText(path, QtGui.QClipboard.Mode.Selection )
        cb.setText(path, QtGui.QClipboard.Mode.Clipboard )

    def IsEnabled(self):
        return hasattr(self._item, "path")

#
# Mute/Unmute the layer represented by this menu item
#
class MuteOrUnmuteLayerMenuItem(LayerStackContextMenuItem):
    def GetText(self):
        stage = getattr(self._item, "stage")
        identifier = getattr(self._item, "identifier")
        return 'Unmute Layer' \
                 if stage.IsLayerMuted(identifier) \
                    else 'Mute Layer'

    def RunCommand(self):
        if not self._item:
            return

        identifier = getattr(self._item, "identifier")
        if not identifier:
            return

        stage = getattr(self._item, "stage")
        
        if not stage.IsLayerMuted(identifier):
            stage.MuteLayer(identifier)
        else:
            stage.UnmuteLayer(identifier)
        
    def IsEnabled(self):
        return True



#
# Copyright 2016 Pixar
#
# Licensed under the terms set forth in the LICENSE.txt file available at
# https://openusd.org/license.
#
from pxr import Tf

# Register this library with a new, previously unknown dependency.  Note, we're
# just using 'sys' as the module to load here, which is a lie.  It should really
# be this module, but it doesn't matter, and it's not trivial to come up with a
# robust, correct module name for this module.
Tf.ScriptModuleLoader()._RegisterLibrary('Unknown', 'sys',
                                         ['NewDynamicDependency'])

# Register the dependency.  In this case we use 'sys' just because it saves us
# from creating a real module called NewDynamicDependency.
Tf.ScriptModuleLoader()._RegisterLibrary('NewDynamicDependency', 'sys', [])

# Load dependencies for this module, which includes the
# NewDynamicDependency. This is a reentrant load that will be handled
# immediately by TfScriptModuleLoader.
Tf.ScriptModuleLoader()._LoadModulesForLibrary('Unknown')





# -*- coding: utf-8 -*-

################################################################################
## Form generated from reading UI file 'attributeValueEditorUI.ui'
##
## Created by: Qt User Interface Compiler version 6.11.2
##
## WARNING! All changes made in this file will be lost when recompiling UI file!
################################################################################

from PySide6.QtCore import (QCoreApplication, QDate, QDateTime, QLocale,
    QMetaObject, QObject, QPoint, QRect,
    QSize, QTime, QUrl, Qt)
from PySide6.QtGui import (QBrush, QColor, QConicalGradient, QCursor,
    QFont, QFontDatabase, QGradient, QIcon,
    QImage, QKeySequence, QLinearGradient, QPainter,
    QPalette, QPixmap, QRadialGradient, QTransform)
from PySide6.QtWidgets import (QApplication, QSizePolicy, QStackedWidget, QTextBrowser,
    QVBoxLayout, QWidget)

class Ui_AttributeValueEditor(object):
    def setupUi(self, AttributeValueEditor):
        if not AttributeValueEditor.objectName():
            AttributeValueEditor.setObjectName(u"AttributeValueEditor")
        AttributeValueEditor.resize(400, 300)
        self.verticalLayout = QVBoxLayout(AttributeValueEditor)
        self.verticalLayout.setObjectName(u"verticalLayout")
        self.verticalLayout.setContentsMargins(0, 0, 0, 0)
        self.stackedWidget = QStackedWidget(AttributeValueEditor)
        self.stackedWidget.setObjectName(u"stackedWidget")
        self.stackedWidget.setLineWidth(0)
        self.valueViewer = QTextBrowser()
        self.valueViewer.setObjectName(u"valueViewer")
        self.stackedWidget.addWidget(self.valueViewer)

        self.verticalLayout.addWidget(self.stackedWidget)


        self.retranslateUi(AttributeValueEditor)

        self.stackedWidget.setCurrentIndex(0)


        QMetaObject.connectSlotsByName(AttributeValueEditor)
    # setupUi

    def retranslateUi(self, AttributeValueEditor):
        AttributeValueEditor.setWindowTitle(QCoreApplication.translate("AttributeValueEditor", u"Form", None))
        AttributeValueEditor.setProperty(u"comment", QCoreApplication.translate("AttributeValueEditor", u"\n"
"     Copyright 2016 Pixar                                                                   \n"
"                                                                                            \n"
"     Licensed under the terms set forth in the LICENSE.txt file available at\n"
"     https://openusd.org/license.\n"
"  ", None))
    # retranslateUi




#
# Copyright 2019 Pixar
#
# Licensed under the terms set forth in the LICENSE.txt file available at
# https://openusd.org/license.
#
from .qt import QtCore, QtGui, QtWidgets

from pxr import Tf


# Returns a list of debug flags whose names are prefixed by debugFlagPrefix
# followed by an '_'.
#
def _GetDebugFlagsWithPrefix(debugFlagPrefix):
    debugFlagPrefix += "_"
    return [flag for flag in Tf.Debug.GetDebugSymbolNames() 
            if flag.startswith(debugFlagPrefix)]
    
# Main DebugFlags editor widget
#
class DebugFlagsWidget(QtWidgets.QWidget):

    def __init__(self, parent=None):

        super(DebugFlagsWidget, self).__init__(parent)

        self.setObjectName("Debug Flags")
        self.setMinimumSize(640, 300)
        self._listView = QtWidgets.QListView()
        self._tableWidget = QtWidgets.QTableWidget(0, 2)


        # Configure and populate the left list view
        self._listView.setEditTriggers(
            QtWidgets.QAbstractItemView.EditTrigger.NoEditTriggers)

        self._populateDebugFlagsListView(self._listView)

        self._listView.selectionModel().selectionChanged.connect(
            self._onFlagSelectionChanged)


        # Configure the table widget
        self._tableWidget.horizontalHeader().setStretchLastSection(True)
        self._tableWidget.horizontalHeader().setDefaultSectionSize(200)
        self._tableWidget.setHorizontalHeaderLabels(
                    ['Debug Symbol', 'Description'])
        self._tableWidget.verticalHeader().hide()

        self._tableWidget.itemClicked.connect(self._onDebugFlagChecked)
        
        # Set the layout
        lay = QtWidgets.QHBoxLayout()
        lay.addWidget(self._listView, 0)
        lay.addWidget(self._tableWidget, 1)
        self.setLayout(lay)

    def _populateDebugFlagsListView(self, listView):
        allDebugFlags = Tf.Debug.GetDebugSymbolNames()
        allDebugPrefixes = [ x[:x.find('_')] if x.find('_') > 0 else x
                for x in allDebugFlags]
        self._allDebugFlagPrefixes = list(sorted(set(allDebugPrefixes)))
        listModel = QtCore.QStringListModel(self._allDebugFlagPrefixes)
        listView.setModel(listModel)

    def _populateDebugFlagsTableView(self, debugFlagPrefix):
        debugFlags = _GetDebugFlagsWithPrefix(debugFlagPrefix)

        self._tableWidget.setRowCount(len(debugFlags))
        row = 0
        for f in debugFlags:
            item = QtWidgets.QTableWidgetItem()
            item.setFlags(QtCore.Qt.ItemFlag.ItemIsUserCheckable | 
                          QtCore.Qt.ItemFlag.ItemIsEnabled)
            if Tf.Debug.IsDebugSymbolNameEnabled(f):
                item.setCheckState(QtCore.Qt.CheckState.Checked)
            else:
                item.setCheckState(QtCore.Qt.CheckState.Unchecked)
            item.setText(f)
            self._tableWidget.setItem(row, 0, item)

            item = QtWidgets.QTableWidgetItem()
            item.setFlags(item.flags() ^ QtCore.Qt.ItemFlag.ItemIsEditable)
            item.setText(Tf.Debug.GetDebugSymbolDescription(f))
            self._tableWidget.setItem(row, 1, item)

            row += 1

    def _onFlagSelectionChanged(self, selected, deselected):
        if len(selected.indexes()) > 0:
            self._populateDebugFlagsTableView(self._allDebugFlagPrefixes[
                selected.indexes()[0].row()])

    def _onDebugFlagChecked(self, item):
        value = (item.checkState() == QtCore.Qt.CheckState.Checked)
        Tf.Debug.SetDebugSymbolsByName(item.text(), value)



# -*- coding: utf-8 -*-

################################################################################
## Form generated from reading UI file 'adjustFreeCameraUI.ui'
##
## Created by: Qt User Interface Compiler version 6.11.2
##
## WARNING! All changes made in this file will be lost when recompiling UI file!
################################################################################

from PySide6.QtCore import (QCoreApplication, QDate, QDateTime, QLocale,
    QMetaObject, QObject, QPoint, QRect,
    QSize, QTime, QUrl, Qt)
from PySide6.QtGui import (QBrush, QColor, QConicalGradient, QCursor,
    QFont, QFontDatabase, QGradient, QIcon,
    QImage, QKeySequence, QLinearGradient, QPainter,
    QPalette, QPixmap, QRadialGradient, QTransform)
from PySide6.QtWidgets import (QApplication, QCheckBox, QDialog, QDoubleSpinBox,
    QHBoxLayout, QLabel, QSizePolicy, QVBoxLayout,
    QWidget)

class Ui_AdjustFreeCamera(object):
    def setupUi(self, AdjustFreeCamera):
        if not AdjustFreeCamera.objectName():
            AdjustFreeCamera.setObjectName(u"AdjustFreeCamera")
        AdjustFreeCamera.resize(331, 140)
        self.verticalLayout = QVBoxLayout(AdjustFreeCamera)
        self.verticalLayout.setObjectName(u"verticalLayout")
        self.horizontalLayout = QHBoxLayout()
        self.horizontalLayout.setObjectName(u"horizontalLayout")
        self.verticalLayout_2 = QVBoxLayout()
        self.verticalLayout_2.setObjectName(u"verticalLayout_2")
        self.overrideNear = QCheckBox(AdjustFreeCamera)
        self.overrideNear.setObjectName(u"overrideNear")
        self.overrideNear.setFocusPolicy(Qt.NoFocus)

        self.verticalLayout_2.addWidget(self.overrideNear)

        self.overrideFar = QCheckBox(AdjustFreeCamera)
        self.overrideFar.setObjectName(u"overrideFar")
        self.overrideFar.setFocusPolicy(Qt.NoFocus)

        self.verticalLayout_2.addWidget(self.overrideFar)


        self.horizontalLayout.addLayout(self.verticalLayout_2)

        self.verticalLayout_3 = QVBoxLayout()
        self.verticalLayout_3.setObjectName(u"verticalLayout_3")
        self.nearSpinBox = QDoubleSpinBox(AdjustFreeCamera)
        self.nearSpinBox.setObjectName(u"nearSpinBox")
        sizePolicy = QSizePolicy(QSizePolicy.Policy.Preferred, QSizePolicy.Policy.Preferred)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.nearSpinBox.sizePolicy().hasHeightForWidth())
        self.nearSpinBox.setSizePolicy(sizePolicy)
        self.nearSpinBox.setDecimals(3)
        self.nearSpinBox.setMinimum(0.000000000000000)
        self.nearSpinBox.setMaximum(1000000000000.000000000000000)
        self.nearSpinBox.setSingleStep(1.000000000000000)

        self.verticalLayout_3.addWidget(self.nearSpinBox)

        self.farSpinBox = QDoubleSpinBox(AdjustFreeCamera)
        self.farSpinBox.setObjectName(u"farSpinBox")
        sizePolicy.setHeightForWidth(self.farSpinBox.sizePolicy().hasHeightForWidth())
        self.farSpinBox.setSizePolicy(sizePolicy)
        self.farSpinBox.setDecimals(3)
        self.farSpinBox.setMinimum(0.000000000000000)
        self.farSpinBox.setMaximum(1000000000000.000000000000000)
        self.farSpinBox.setSingleStep(1.000000000000000)

        self.verticalLayout_3.addWidget(self.farSpinBox)


        self.horizontalLayout.addLayout(self.verticalLayout_3)


        self.verticalLayout.addLayout(self.horizontalLayout)

        self.horizontalLayout_2 = QHBoxLayout()
        self.horizontalLayout_2.setObjectName(u"horizontalLayout_2")
        self.lockFreeCamAspect = QCheckBox(AdjustFreeCamera)
        self.lockFreeCamAspect.setObjectName(u"lockFreeCamAspect")
        self.lockFreeCamAspect.setFocusPolicy(Qt.NoFocus)

        self.horizontalLayout_2.addWidget(self.lockFreeCamAspect)

        self.freeCamAspect = QDoubleSpinBox(AdjustFreeCamera)
        self.freeCamAspect.setObjectName(u"freeCamAspect")
        self.freeCamAspect.setDecimals(3)
        self.freeCamAspect.setMinimum(0.000000000000000)
        self.freeCamAspect.setSingleStep(0.100000000000000)

        self.horizontalLayout_2.addWidget(self.freeCamAspect)


        self.verticalLayout.addLayout(self.horizontalLayout_2)

        self.horizontalLayout_3 = QHBoxLayout()
        self.horizontalLayout_3.setObjectName(u"horizontalLayout_3")
        self.freeCamFovLabel = QLabel(AdjustFreeCamera)
        self.freeCamFovLabel.setObjectName(u"freeCamFovLabel")
        self.freeCamFovLabel.setFocusPolicy(Qt.NoFocus)

        self.horizontalLayout_3.addWidget(self.freeCamFovLabel)

        self.freeCamFov = QDoubleSpinBox(AdjustFreeCamera)
        self.freeCamFov.setObjectName(u"freeCamFov")
        self.freeCamFov.setDecimals(3)
        self.freeCamFov.setMinimum(0.000000000000000)
        self.freeCamFov.setMaximum(180.000000000000000)
        self.freeCamFov.setSingleStep(1.000000000000000)

        self.horizontalLayout_3.addWidget(self.freeCamFov)


        self.verticalLayout.addLayout(self.horizontalLayout_3)


        self.retranslateUi(AdjustFreeCamera)

        QMetaObject.connectSlotsByName(AdjustFreeCamera)
    # setupUi

    def retranslateUi(self, AdjustFreeCamera):
        AdjustFreeCamera.setWindowTitle(QCoreApplication.translate("AdjustFreeCamera", u"Adjust Free Camera", None))
        AdjustFreeCamera.setProperty(u"comment", QCoreApplication.translate("AdjustFreeCamera", u"\n"
"     Copyright 2016 Pixar                                                                   \n"
"                                                                                            \n"
"     Licensed under the terms set forth in the LICENSE.txt file available at\n"
"     https://openusd.org/license.\n"
"  ", None))
        self.overrideNear.setText(QCoreApplication.translate("AdjustFreeCamera", u"Override Near", None))
        self.overrideFar.setText(QCoreApplication.translate("AdjustFreeCamera", u"Override Far", None))
        self.lockFreeCamAspect.setText(QCoreApplication.translate("AdjustFreeCamera", u"Aspect Ratio", None))
        self.freeCamFovLabel.setText(QCoreApplication.translate("AdjustFreeCamera", u"FOV (degrees)", None))
    # retranslateUi




#
# Copyright 2016 Pixar
#
# Licensed under the terms set forth in the LICENSE.txt file available at
# https://openusd.org/license.
#
from pxr import Tf

raise RuntimeError('This module should not get loaded!')





# -*- coding: utf-8 -*-

################################################################################
## Form generated from reading UI file 'adjustDefaultMaterialUI.ui'
##
## Created by: Qt User Interface Compiler version 6.11.2
##
## WARNING! All changes made in this file will be lost when recompiling UI file!
################################################################################

from PySide6.QtCore import (QCoreApplication, QDate, QDateTime, QLocale,
    QMetaObject, QObject, QPoint, QRect,
    QSize, QTime, QUrl, Qt)
from PySide6.QtGui import (QBrush, QColor, QConicalGradient, QCursor,
    QFont, QFontDatabase, QGradient, QIcon,
    QImage, QKeySequence, QLinearGradient, QPainter,
    QPalette, QPixmap, QRadialGradient, QTransform)
from PySide6.QtWidgets import (QApplication, QDialog, QDoubleSpinBox, QHBoxLayout,
    QLabel, QPushButton, QSizePolicy, QSpacerItem,
    QVBoxLayout, QWidget)

class Ui_AdjustDefaultMaterial(object):
    def setupUi(self, AdjustDefaultMaterial):
        if not AdjustDefaultMaterial.objectName():
            AdjustDefaultMaterial.setObjectName(u"AdjustDefaultMaterial")
        AdjustDefaultMaterial.resize(280, 123)
        sizePolicy = QSizePolicy(QSizePolicy.Policy.Minimum, QSizePolicy.Policy.Minimum)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(AdjustDefaultMaterial.sizePolicy().hasHeightForWidth())
        AdjustDefaultMaterial.setSizePolicy(sizePolicy)
        self.verticalLayout = QVBoxLayout(AdjustDefaultMaterial)
        self.verticalLayout.setObjectName(u"verticalLayout")
        self.verticalLayout_4 = QVBoxLayout()
        self.verticalLayout_4.setObjectName(u"verticalLayout_4")
        self.horizontalLayout = QHBoxLayout()
        self.horizontalLayout.setObjectName(u"horizontalLayout")
        self.horizontalSpacer_4 = QSpacerItem(40, 20, QSizePolicy.Policy.Expanding, QSizePolicy.Policy.Minimum)

        self.horizontalLayout.addItem(self.horizontalSpacer_4)

        self.ambientInt = QLabel(AdjustDefaultMaterial)
        self.ambientInt.setObjectName(u"ambientInt")
        self.ambientInt.setFocusPolicy(Qt.NoFocus)

        self.horizontalLayout.addWidget(self.ambientInt)

        self.ambientIntSpinBox = QDoubleSpinBox(AdjustDefaultMaterial)
        self.ambientIntSpinBox.setObjectName(u"ambientIntSpinBox")
        self.ambientIntSpinBox.setDecimals(1)
        self.ambientIntSpinBox.setMaximum(1.000000000000000)
        self.ambientIntSpinBox.setSingleStep(0.100000000000000)

        self.horizontalLayout.addWidget(self.ambientIntSpinBox)

        self.horizontalSpacer_5 = QSpacerItem(40, 20, QSizePolicy.Policy.Expanding, QSizePolicy.Policy.Minimum)

        self.horizontalLayout.addItem(self.horizontalSpacer_5)


        self.verticalLayout_4.addLayout(self.horizontalLayout)

        self.horizontalLayout_2 = QHBoxLayout()
        self.horizontalLayout_2.setObjectName(u"horizontalLayout_2")
        self.horizontalSpacer_6 = QSpacerItem(40, 20, QSizePolicy.Policy.Expanding, QSizePolicy.Policy.Minimum)

        self.horizontalLayout_2.addItem(self.horizontalSpacer_6)

        self.specularInt = QLabel(AdjustDefaultMaterial)
        self.specularInt.setObjectName(u"specularInt")
        self.specularInt.setFocusPolicy(Qt.NoFocus)

        self.horizontalLayout_2.addWidget(self.specularInt)

        self.specularIntSpinBox = QDoubleSpinBox(AdjustDefaultMaterial)
        self.specularIntSpinBox.setObjectName(u"specularIntSpinBox")
        self.specularIntSpinBox.setDecimals(1)
        self.specularIntSpinBox.setMaximum(1.000000000000000)
        self.specularIntSpinBox.setSingleStep(0.100000000000000)

        self.horizontalLayout_2.addWidget(self.specularIntSpinBox)

        self.horizontalSpacer_7 = QSpacerItem(40, 20, QSizePolicy.Policy.Expanding, QSizePolicy.Policy.Minimum)

        self.horizontalLayout_2.addItem(self.horizontalSpacer_7)


        self.verticalLayout_4.addLayout(self.horizontalLayout_2)

        self.horizontalLayout_3 = QHBoxLayout()
        self.horizontalLayout_3.setObjectName(u"horizontalLayout_3")
        self.resetButton = QPushButton(AdjustDefaultMaterial)
        self.resetButton.setObjectName(u"resetButton")
        self.resetButton.setAutoDefault(False)

        self.horizontalLayout_3.addWidget(self.resetButton)

        self.horizontalSpacer_2 = QSpacerItem(40, 20, QSizePolicy.Policy.Expanding, QSizePolicy.Policy.Minimum)

        self.horizontalLayout_3.addItem(self.horizontalSpacer_2)

        self.doneButton = QPushButton(AdjustDefaultMaterial)
        self.doneButton.setObjectName(u"doneButton")

        self.horizontalLayout_3.addWidget(self.doneButton)


        self.verticalLayout_4.addLayout(self.horizontalLayout_3)


        self.verticalLayout.addLayout(self.verticalLayout_4)


        self.retranslateUi(AdjustDefaultMaterial)

        QMetaObject.connectSlotsByName(AdjustDefaultMaterial)
    # setupUi

    def retranslateUi(self, AdjustDefaultMaterial):
        AdjustDefaultMaterial.setWindowTitle(QCoreApplication.translate("AdjustDefaultMaterial", u"Adjust Default Material", None))
        AdjustDefaultMaterial.setProperty(u"comment", QCoreApplication.translate("AdjustDefaultMaterial", u"\n"
"     Copyright 2017 Pixar                                                                   \n"
"                                                                                            \n"
"     Licensed under the terms set forth in the LICENSE.txt file available at\n"
"     https://openusd.org/license.\n"
"  ", None))
        self.ambientInt.setText(QCoreApplication.translate("AdjustDefaultMaterial", u"Ambient Intensity", None))
        self.specularInt.setText(QCoreApplication.translate("AdjustDefaultMaterial", u"Specular Intensity", None))
        self.resetButton.setText(QCoreApplication.translate("AdjustDefaultMaterial", u"Reset", None))
        self.doneButton.setText(QCoreApplication.translate("AdjustDefaultMaterial", u"Done", None))
    # retranslateUi




#!/pxrpythonsubst
#
# Copyright 2025 Pixar
#
# Licensed under the terms set forth in the LICENSE.txt file available at
# https://openusd.org/license.
#
"""
This script helps set up build necessities for schema libraries. It will 
generate templates for the following files if they do not already exist:
 * CMakeLists.txt 
 * __init__.py
 * schema.usda
 * module.cpp
 * schemaUserDoc.usda

"""

import sys, os, inspect
from argparse import ArgumentParser

from jinja2 import Environment, FileSystemLoader
from jinja2.exceptions import TemplateNotFound, TemplateSyntaxError

from pxr import Plug
from pxr.Usd.usdGenSchema import _Printer, _WriteFile

Print = _Printer()

# Name of script, e.g. "usdInitSchema"
PROGRAM_NAME = os.path.splitext(os.path.basename(sys.argv[0]))[0]

def _WriteFileIfNotPresent(fileName, filePath, renderedTemplate, validate):
    if os.path.exists(filePath):
        Print("\t{0} is already present, skipping generation.".format(fileName))
    else:
        _WriteFile(filePath, renderedTemplate, validate)    

if __name__ == '__main__':
    #
    # Parse Command-line
    #
    parser = ArgumentParser(description='Set up build necessities for schema classes. This will generate '
             'templates for the CMakeLists.txt, __init__.py, schema.usda, '
             'module.cpp, and a user doc if they do not already exist.')
    parser.add_argument('codeGenPath',
        type=str,
        help='The target directory where the code should be generated. '
        'Can be \'.\' for the current directory.')
    parser.add_argument('-v', '--validate',
        action='store_true',
        help='Verify that the source files are unchanged.')
    parser.add_argument('-q', '--quiet',
        action='store_true',
        help='Do not output text during execution.')
    parser.add_argument('libraryPath',
        type=str,
        help='The partial path with which to prefix \'#include\' statements of '
             'generated files for this module. For external (non-Pixar) '
             'plugins, we recommend setting libraryPath to ".", so that the '
             'headers inside src directory are included in the generated files.')
    parser.add_argument('pxrPrefix',
        nargs='?',
        type=str,
        default="",
        help='Prefix to the path from pxr to this library. For pxr/usd/usd, '
             'the prefix would be pxr/usd. Omit this argument for non-OpenUSD '
             'build systems.')
    parser.add_argument('--genModuleDeps',
        action='store_true',
        help='Generates a template for the moduleDeps.cpp. This template will '
             'need to be filled in and kept up to date with library '
             'dependencies if you are not using the OpenUSD build system.')

    defaultTemplateDir = os.path.join(
        os.path.dirname(
            os.path.abspath(inspect.getfile(inspect.currentframe()))),
        'codegenTemplates')

    instTemplateDir = os.path.join(
        os.path.abspath(Plug.Registry().GetPluginWithName('usd').resourcePath),
        'codegenTemplates')

    parser.add_argument('-t', '--templates',
        dest='templatePath',
        type=str,
        help=('Directory containing schema class templates. '
              '[Default: first directory that exists from this list: {0}]'
              .format(os.pathsep.join([defaultTemplateDir, instTemplateDir]))))

    args = parser.parse_args()
    codeGenPath = os.path.abspath(args.codeGenPath)
    libraryName = os.path.basename(codeGenPath)

    if args.templatePath:
        templatePath = os.path.abspath(args.templatePath)
    else:
        if os.path.isdir(defaultTemplateDir):
            templatePath = defaultTemplateDir
        else:
            templatePath = instTemplateDir

    Print.SetQuiet(args.quiet)

    env = Environment(loader=FileSystemLoader(templatePath),
                            trim_blocks=True)

    # File names
    initFileName = '__init__.py'
    moduleFileName = 'module.cpp'
    schemaFileName = 'schema.usda'
    schemaDocFileName = 'schemaUserDoc.usda'
    cmakeFileName = 'CMakeLists.txt'
    genModuleFileName = 'generatedSchema.module.h'
    genClassesFileName = 'generatedSchema.classes.txt'
    moduleDepsFileName = 'moduleDeps.cpp'

    # Load templates
    Print('Loading Templates from {0}'.format(templatePath))
    try:
        initTemplate = env.get_template(initFileName)
        moduleTemplate = env.get_template(moduleFileName)
        schemaTemplate = env.get_template(schemaFileName)
        schemaDocTemplate = env.get_template(schemaDocFileName)
        cmakeTemplate = env.get_template(cmakeFileName)
        genModuleTemplate = env.get_template(genModuleFileName)
        genClassesTemplate = env.get_template(genClassesFileName)
        if args.genModuleDeps:
            genModuleDepsTemplate = env.get_template(moduleDepsFileName)
    except TemplateNotFound as tnf:
        raise RuntimeError("Template not found: {0}".format(str(tnf)))
    except TemplateSyntaxError as tse:
        raise RuntimeError("Syntax error in template {0} at line {1}: {2}"
                        .format(tse.filename, tse.lineno, tse))

    # Create directories for the codegen and user docs
    if not os.path.exists(os.path.dirname(codeGenPath)):
        os.makedirs(os.path.dirname(codeGenPath))

    userDocFolderPath = os.path.join(codeGenPath, 'userDoc')
    if not os.path.exists(userDocFolderPath):
        os.makedirs(userDocFolderPath)

    # Generate an __init__.py if not present
    initPath = os.path.join(codeGenPath, initFileName)
    _WriteFileIfNotPresent(initFileName, initPath, initTemplate.render(), args.validate)

    # Generate a schema.usda if not present
    schemaPath = os.path.join(codeGenPath, schemaFileName)
    _WriteFileIfNotPresent(schemaFileName, schemaPath,
                           schemaTemplate.render(libraryName=libraryName, 
                                                libraryPath=args.libraryPath), args.validate)

    # Generate a schema doc if not present
    schemaDocPath = os.path.join(userDocFolderPath, schemaDocFileName)
    _WriteFileIfNotPresent(schemaDocFileName, schemaDocPath, 
                           schemaDocTemplate.render(), args.validate)

    # Generate a module.cpp if not present
    filePath = os.path.join(codeGenPath, moduleFileName)
    if os.path.exists(filePath):
        # Check whether the module.cpp includes the generated module file
        with open(filePath, 'r') as f:
            foundModuleCpp = False
            for line in f.readlines():
                if "#include \"" + genModuleFileName + "\"" in line.strip():
                    foundModuleCpp = True
            
            if not foundModuleCpp:
                Print("WARNING:", genModuleFileName, "not included in "
                    "module.cpp. This should be included to avoid "
                    "listing entries manually.")
    else:        
        _WriteFile(filePath, moduleTemplate.render(), args.validate)

    # Generate a moduleDeps.cpp if requested and not present
    if args.genModuleDeps:
        moduleDepsPath = os.path.join(codeGenPath, moduleDepsFileName)
        _WriteFileIfNotPresent(moduleDepsFileName, moduleDepsPath, 
                               genModuleDepsTemplate.render(), args.validate)

    # Generate empty files to be filled in by usdGenSchema after the schema.usda
    # is filled in
    _WriteFileIfNotPresent(genModuleFileName, os.path.join(codeGenPath, 
                genModuleFileName), genModuleTemplate.render(), args.validate)
    _WriteFileIfNotPresent(genClassesFileName, os.path.join(codeGenPath, 
                genClassesFileName), genClassesTemplate.render(), args.validate)

    # Check whether the CMakeLists.txt call to pxr_library includes the 
    # INCLUDE_SCHEMA_FILES flag
    cmakePath = os.path.join(codeGenPath, cmakeFileName)
    if os.path.exists(cmakePath):
        with open(cmakePath, 'r') as f:
            if "INCLUDE_SCHEMA_FILES" not in f.read():
                Print("WARNING: INCLUDE_SCHEMA_FLAGS not present in pxr_library "
                      "in ", cmakeFileName, ". Pass this flag to pxr_library to "
                      "have schema classes automatically built.")
    else:
        _WriteFile(cmakePath, cmakeTemplate.render(pxrPrefix=args.pxrPrefix, 
                    libraryName=libraryName), args.validate)
    
